@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -15,6 +15,7 @@
 #include "VerifyFormat.h"
 #include "StringXStream.h"
 #include "UTF8Conv.h"
+#include "PWPolicy.h"
 
 #include "os/typedefs.h"
 #include "os/debug.h"
@@ -22,9 +23,11 @@
 #include "os/file.h"
 #include "os/dir.h"
 #include "os/registry.h"
+#include "os/logit.h"
 
 #include <fstream>
 #include <algorithm>
+#include <iomanip>
 
 #ifdef _WIN32
 #include <LMCons.h> // for UNLEN
@@ -32,13 +35,8 @@
 
 using namespace std;
 
-#if defined(POCKET_PC)
-const LPCTSTR PWS_REG_POSITION = _T("Position");
-const LPCTSTR PWS_REG_OPTIONS = _T("Options");
-#else
 const LPCTSTR PWS_REG_POSITION = _T("");
 const LPCTSTR PWS_REG_OPTIONS = _T("");
-#endif
 
 // For Old registry handling:
 const stringT Software(_T("Software"));
@@ -57,7 +55,7 @@ bool PWSprefs::m_userSetCfgFile = false; // true iff user set config file (-g co
 const stringT PWSprefs::cfgFileName = _T("pwsafe.cfg");
 
 /*
- Note: to change a preference between application and database, the way 
+ Note: to change a preference between application and database, the way
  to do it is to set the current one as obsolete and define a new one.
 
  Do not change the current type.
@@ -124,7 +122,9 @@ const PWSprefs::boolPref PWSprefs::m_bool_prefs[NumBoolPrefs] = {
   {_T("HighlightChanges"), true, ptApplication},            // application
   {_T("HideSystemTray"), false, ptApplication},             // application
   {_T("UsePrimarySelectionForClipboard"), false, ptApplication}, //application
-  {_T("CopyPasswordWhenBrowseToURL"), true, ptDatabase},    // database
+  {_T("CopyPasswordWhenBrowseToURL"), false, ptDatabase},   // database
+  {_T("UseAltAutoType"), false, ptApplication},             //application
+  {_T("UseSecureDesktop"), false, ptApplication},          //application
 };
 
 // Default value = -1 means set at runtime
@@ -148,8 +148,8 @@ const PWSprefs::intPref PWSprefs::m_int_prefs[NumIntPrefs] = {
                                   minTDS, maxTDS},                  // database
   {_T("NumPWHistoryDefault"), 3, ptDatabase, 0, 255},               // database
   // Specified by supported masks
-  {_T("BackupSuffix"), BKSFX_None, ptApplication, minBKSFX, maxBKSFX}, // application
-  {_T("BackupMaxIncremented"), 1, ptApplication, 1, 999},           // application
+  {_T("BackupSuffix"), BKSFX_IncNumber, ptApplication, minBKSFX, maxBKSFX}, // application
+  {_T("BackupMaxIncremented"), 3, ptApplication, 1, 999},           // application
   {_T("PreExpiryWarnDays"), 1, ptApplication, 1, 30},               // application
   {_T("ClosedTrayIconColour"), stiBlack, ptApplication,
                                stiBlack, stiYellow},                // application
@@ -158,6 +158,11 @@ const PWSprefs::intPref PWSprefs::m_int_prefs[NumIntPrefs] = {
   {_T("PWSymbolMinLength"), 0, ptDatabase, 0, 1024},                // database
   {_T("PWUppercaseMinLength"), 0, ptDatabase, 0, 1024},             // database
   {_T("OptShortcutColumnWidth"), 92, ptApplication, 10, 512},       // application
+  {_T("ShiftDoubleClickAction"), DoubleClickCopyUsername, ptApplication,
+                            minDCA, maxDCA},                        // application
+  {_T("DefaultAutotypeDelay"), 10, ptApplication,
+                            1, 60000},                              // application
+  {_T("SecureDesktopTimeout"), 120, ptApplication, 60, 300},        // application
 };
 
 const PWSprefs::stringPref PWSprefs::m_string_prefs[NumStringPrefs] = {
@@ -177,7 +182,7 @@ const PWSprefs::stringPref PWSprefs::m_string_prefs[NumStringPrefs] = {
   {_T("PasswordFont"), _T(""), ptApplication},                      // application
   {_T("TreeListSampleText"), _T("AaBbYyZz 0O1IlL"), ptApplication}, // application
   {_T("PswdSampleText"), _T("AaBbYyZz 0O1IlL"), ptApplication},     // application
-  {_T("LastUsedKeyboard"), _T(""), ptApplication},                  // application 
+  {_T("LastUsedKeyboard"), _T(""), ptApplication},                  // application
   {_T("VKeyboardFontName"), _T(""), ptApplication},                 // application
   {_T("VKSampleText"), _T("AaBbYyZz 0O1IlL"), ptApplication},       // application
   {_T("AltNotesEditor"), _T(""), ptApplication},                    // application
@@ -202,12 +207,9 @@ void PWSprefs::DeleteInstance()
 
 bool PWSprefs::SetConfigFile(const stringT &fn)
 {
-  if (pws_os::FileExists(fn)) {
-    m_configfilename = fn;
-    m_userSetCfgFile = true;
-    return true;
-  } else
-    return false;
+  m_configfilename = fn;
+  m_userSetCfgFile = true;
+  return pws_os::FileExists(fn);
 }
 
 PWSprefs::PWSprefs() : m_pXML_Config(NULL)
@@ -240,19 +242,19 @@ bool PWSprefs::CheckRegistryExists() const
   return pws_os::RegCheckExists();
 }
 
-bool PWSprefs::GetPref(BoolPrefs pref_enum) const
+bool PWSprefs::GetPref(BoolPrefs pref_enum, const bool bUseCopy) const
 {
-  return m_boolValues[pref_enum];
+  return bUseCopy ? m_boolCopyValues[pref_enum] : m_boolValues[pref_enum];
 }
 
-unsigned int PWSprefs::GetPref(IntPrefs pref_enum) const
+unsigned int PWSprefs::GetPref(IntPrefs pref_enum, const bool bUseCopy) const
 {
-  return m_intValues[pref_enum];
+  return bUseCopy ? m_intCopyValues[pref_enum] : m_intValues[pref_enum];
 }
 
-StringX PWSprefs::GetPref(StringPrefs pref_enum) const
+StringX PWSprefs::GetPref(StringPrefs pref_enum, const bool bUseCopy) const
 {
-  return m_stringValues[pref_enum];
+  return bUseCopy ? m_stringCopyValues[pref_enum] : m_stringValues[pref_enum];
 }
 
 bool PWSprefs::GetPrefDefVal(BoolPrefs pref_enum) const
@@ -270,10 +272,85 @@ StringX PWSprefs::GetPrefDefVal(StringPrefs pref_enum) const
   return m_string_prefs[pref_enum].defVal;
 }
 
-// Following for case where default value is determined at runtime
-unsigned int PWSprefs::GetPref(IntPrefs pref_enum, unsigned int defVal) const
+StringX PWSprefs::GetAllBoolPrefs(const bool bUseCopy)
 {
-  return m_intValues[pref_enum] == static_cast<unsigned int>(-1) ? defVal : m_intValues[pref_enum];
+  PWS_LOGIT;
+
+  oStringXStream osxs;
+  for (int i = 0; i < NumBoolPrefs; i++) {
+    osxs << setw(1) << i << TCHAR(' ')
+         << setw(1) << m_bool_prefs[i].ptype << TCHAR(' ')
+         << setw(1) <<
+         ((bUseCopy ? m_boolCopyValues[i] : m_boolValues[i]) ? 1 : 0) << TCHAR(' ');
+  }
+  return osxs.str();
+}
+
+StringX PWSprefs::GetAllIntPrefs(const bool bUseCopy)
+{
+  PWS_LOGIT;
+
+  oStringXStream osxs;
+  for (int i = 0; i < NumIntPrefs; i++) {
+    osxs << setw(1) << i << TCHAR(' ')
+         << setw(1) << m_int_prefs[i].ptype << TCHAR(' ')
+         << setw(4) << setfill(TCHAR('0')) << hex <<
+         (bUseCopy ? m_intCopyValues[i] : m_intValues[i]) << dec << TCHAR(' ');
+  }
+  return osxs.str();
+}
+
+StringX PWSprefs::GetAllStringPrefs(const bool bUseCopy)
+{
+  PWS_LOGIT;
+
+  // Here we must restrict most string preferences as they contain user data
+  int SafeStringPrefs[] = {
+    LastView, TreeFont, BackupPrefixValue, ListColumns,
+    ColumnWidths, MainToolBarButtons, PasswordFont,
+    TreeListSampleText, PswdSampleText,
+    LastUsedKeyboard, VKeyboardFontName, VKSampleText, LanguageFile
+  };
+
+  TCHAR delim;
+  const TCHAR Delimiters[] = _T("\"\'#?!%&*+=:;@~<>?,.{}[]()\xab\xbb");
+  const size_t NumDelimiters = sizeof(Delimiters) / sizeof(Delimiters[0]) - 1;
+
+  oStringXStream osxs;
+  for (size_t i = 0; i < sizeof(SafeStringPrefs) / sizeof(SafeStringPrefs[0]); i++) {
+    const int k = SafeStringPrefs[i];
+    delim = TCHAR(' ');
+    for (size_t j = 0; j < NumDelimiters; j++) {
+      if (bUseCopy) {
+        if (m_stringCopyValues[k].find(Delimiters[j]) == StringX::npos) {
+          delim = Delimiters[j];
+          break;
+        }
+      } else {
+        if (m_stringValues[k].find(Delimiters[j]) == StringX::npos) {
+          delim = Delimiters[j];
+          break;
+        }
+      }
+    }
+    if (delim == TCHAR(' '))
+      continue;  // We tried, but just can't save it!
+
+    osxs << setw(1) << k << TCHAR(' ')
+         << setw(1) << m_string_prefs[k].ptype << TCHAR(' ')
+         << delim <<
+         (bUseCopy ? m_stringCopyValues[k].c_str() : m_stringValues[k].c_str()) <<
+         delim << TCHAR(' ');
+  }
+  return osxs.str();
+}
+
+// Following for case where default value is determined at runtime
+unsigned int PWSprefs::GetPref(IntPrefs pref_enum, unsigned int defVal,
+                               const bool bUseCopy) const
+{
+  return m_intValues[pref_enum] == static_cast<unsigned int>(-1) ? defVal :
+      (bUseCopy ? m_intCopyValues[pref_enum] : m_intValues[pref_enum]);
 }
 
 void PWSprefs::GetPrefRect(long &top, long &bottom,
@@ -294,7 +371,7 @@ void PWSprefs::GetPrefPSSRect(long &top, long &bottom,
   right = m_PSSrect.right;
 }
 
-int PWSprefs::GetMRUList(stringT *MRUFiles)
+int PWSprefs::GetMRUList(stringT *MRUFiles) const
 {
   ASSERT(MRUFiles != NULL);
 
@@ -333,7 +410,7 @@ int PWSprefs::SetMRUList(const stringT *MRUFiles, int n, int max_MRU)
     }
     cnt++;
   }
-  // Remove any not in use    
+  // Remove any not in use
   for (i = cnt - 1; i < max_MRU; i++) {
     if (!m_MRUitems[i].empty()) {
       m_MRUitems[i] = _T("");
@@ -345,6 +422,62 @@ int PWSprefs::SetMRUList(const stringT *MRUFiles, int n, int max_MRU)
   return n;
 }
 
+PWPolicy PWSprefs::GetDefaultPolicy(const bool bUseCopy) const
+{
+  PWPolicy pwp;
+  if (GetPref(PWUseLowercase, bUseCopy))
+    pwp.flags |= PWPolicy::UseLowercase;
+  if (GetPref(PWUseUppercase, bUseCopy))
+    pwp.flags |= PWPolicy::UseUppercase;
+  if (GetPref(PWUseDigits, bUseCopy))
+    pwp.flags |= PWPolicy::UseDigits;
+  if (GetPref(PWUseSymbols, bUseCopy))
+    pwp.flags |= PWPolicy::UseSymbols;
+  if (GetPref(PWUseHexDigits, bUseCopy))
+    pwp.flags |= PWPolicy::UseHexDigits;
+  if (GetPref(PWUseEasyVision, bUseCopy))
+    pwp.flags |= PWPolicy::UseEasyVision;
+  if (GetPref(PWMakePronounceable, bUseCopy))
+    pwp.flags |= PWPolicy::MakePronounceable;
+
+  pwp.length = GetPref(PWDefaultLength, bUseCopy);
+  pwp.digitminlength = GetPref(PWDigitMinLength, bUseCopy);
+  pwp.lowerminlength = GetPref(PWLowercaseMinLength, bUseCopy);
+  pwp.symbolminlength = GetPref(PWSymbolMinLength, bUseCopy);
+  pwp.upperminlength = GetPref(PWUppercaseMinLength, bUseCopy);
+  pwp.symbols = GetPref(DefaultSymbols, bUseCopy);
+  pwp.Normalize();
+  return pwp;
+}
+
+void PWSprefs::SetDefaultPolicy(const PWPolicy &pol, const bool bUseCopy)
+{
+  PWPolicy nc_pol(pol); // non-const copy that we can Normalize;
+  nc_pol.Normalize();
+
+  SetPref(PWUseLowercase,
+          (nc_pol.flags & PWPolicy::UseLowercase) != 0, bUseCopy);
+  SetPref(PWUseUppercase,
+          (nc_pol.flags & PWPolicy::UseUppercase) != 0, bUseCopy);
+  SetPref(PWUseDigits,
+          (nc_pol.flags & PWPolicy::UseDigits) != 0, bUseCopy);
+  SetPref(PWUseSymbols,
+          (nc_pol.flags & PWPolicy::UseSymbols) != 0, bUseCopy);
+  SetPref(PWUseHexDigits,
+          (nc_pol.flags & PWPolicy::UseHexDigits) != 0, bUseCopy);
+  SetPref(PWUseEasyVision,
+          (nc_pol.flags & PWPolicy::UseEasyVision) != 0, bUseCopy);
+  SetPref(PWMakePronounceable,
+          (nc_pol.flags & PWPolicy::MakePronounceable) != 0, bUseCopy);
+
+  SetPref(PWDefaultLength, nc_pol.length, bUseCopy);
+  SetPref(PWDigitMinLength, nc_pol.digitminlength, bUseCopy);
+  SetPref(PWLowercaseMinLength, nc_pol.lowerminlength, bUseCopy);
+  SetPref(PWSymbolMinLength, nc_pol.symbolminlength, bUseCopy);
+  SetPref(PWUppercaseMinLength, nc_pol.upperminlength, bUseCopy);
+  SetPref(DefaultSymbols, nc_pol.symbols, bUseCopy);
+}
+
 void PWSprefs::SetupCopyPrefs()
 {
   // Set up copy of current preferences
@@ -363,9 +496,11 @@ void PWSprefs::SetupCopyPrefs()
 
 void PWSprefs::UpdateFromCopyPrefs(const PWSprefs::PrefType ptype)
 {
+  PWS_LOGIT;
+
   // Update real preferences from copy values
   for (int i = 0; i < NumBoolPrefs; i++) {
-    if ((ptype & m_bool_prefs[i].ptype) == m_bool_prefs[i].ptype) {
+    if (ptype == ptAll || m_bool_prefs[i].ptype == ptype) {
       // ONLY save in memory - written out at database save (to database and config destination)
       m_prefs_changed[m_bool_prefs[i].ptype == ptDatabase ? DB_PREF : APP_PREF] |=
                                (m_boolValues[i] != m_boolCopyValues[i]);
@@ -378,7 +513,7 @@ void PWSprefs::UpdateFromCopyPrefs(const PWSprefs::PrefType ptype)
   }
 
   for (int i = 0; i < NumIntPrefs; i++) {
-    if ((ptype & m_int_prefs[i].ptype) == m_int_prefs[i].ptype) {
+    if (ptype == ptAll || m_int_prefs[i].ptype == ptype) {
       // ONLY save in memory - written out at database save (to database and config destination)
       m_prefs_changed[m_int_prefs[i].ptype == ptDatabase ? DB_PREF : APP_PREF] |=
                                (m_intValues[i] != m_intCopyValues[i]);
@@ -391,7 +526,7 @@ void PWSprefs::UpdateFromCopyPrefs(const PWSprefs::PrefType ptype)
   }
 
   for (int i = 0; i < NumStringPrefs; i++) {
-    if ((ptype && m_string_prefs[i].ptype) == m_string_prefs[i].ptype) {
+    if (ptype == ptAll || m_string_prefs[i].ptype == ptype) {
       // ONLY save in memory - written out at database save (to database and config destination)
       m_prefs_changed[m_string_prefs[i].ptype == ptDatabase ? DB_PREF : APP_PREF] |=
                                (m_stringValues[i] != m_stringCopyValues[i]);
@@ -460,6 +595,8 @@ void PWSprefs::SetPref(StringPrefs pref_enum, const StringX &value, const bool b
 
 void PWSprefs::ResetPref(BoolPrefs pref_enum)
 {
+  PWS_LOGIT_ARGS("BoolPref: %s", m_bool_prefs[pref_enum].name);
+
   m_boolValues[pref_enum] = m_bool_prefs[pref_enum].defVal;
   m_boolChanged[pref_enum] = true;
   m_prefs_changed[m_bool_prefs[pref_enum].ptype == ptDatabase ? DB_PREF : APP_PREF] = true;
@@ -467,6 +604,8 @@ void PWSprefs::ResetPref(BoolPrefs pref_enum)
 
 void PWSprefs::ResetPref(IntPrefs pref_enum)
 {
+  PWS_LOGIT_ARGS("IntegerPref: %s", m_int_prefs[pref_enum].name);
+
   m_intValues[pref_enum] = m_int_prefs[pref_enum].defVal;
   m_intChanged[pref_enum] = true;
   m_prefs_changed[m_int_prefs[pref_enum].ptype == ptDatabase ? DB_PREF : APP_PREF] = true;
@@ -474,6 +613,8 @@ void PWSprefs::ResetPref(IntPrefs pref_enum)
 
 void PWSprefs::ResetPref(StringPrefs pref_enum)
 {
+  PWS_LOGIT_ARGS("StringPref: %s", m_string_prefs[pref_enum].name);
+
   m_stringValues[pref_enum] = m_string_prefs[pref_enum].defVal;
   m_stringChanged[pref_enum] = true;
   m_prefs_changed[m_string_prefs[pref_enum].ptype == ptDatabase ? DB_PREF : APP_PREF] = true;
@@ -481,6 +622,8 @@ void PWSprefs::ResetPref(StringPrefs pref_enum)
 
 bool PWSprefs::WritePref(const StringX &name, bool val)
 {
+  PWS_LOGIT_ARGS("Bool Name: %s", name.c_str());
+
   // Used to save to config destination at database save and application termination
   bool bRetVal(false);
   switch (m_ConfigOption) {
@@ -502,6 +645,8 @@ bool PWSprefs::WritePref(const StringX &name, bool val)
 
 bool PWSprefs::WritePref(const StringX &name, unsigned int val)
 {
+  PWS_LOGIT_ARGS("Integer Name: %s", name.c_str());
+
   // Used to save to config destination at database save and application termination
   bool bRetVal(false);
   switch (m_ConfigOption) {
@@ -522,6 +667,8 @@ bool PWSprefs::WritePref(const StringX &name, unsigned int val)
 
 bool PWSprefs::WritePref(const StringX &name, const StringX &val)
 {
+  PWS_LOGIT_ARGS("String Name: %s", name.c_str());
+
   // Used to save to config destination at database save and application termination
   bool bRetVal(false);
   switch (m_ConfigOption) {
@@ -543,6 +690,8 @@ bool PWSprefs::WritePref(const StringX &name, const StringX &val)
 
 bool PWSprefs::DeletePref(const StringX &name)
 {
+  PWS_LOGIT_ARGS("Name: %s", name.c_str());
+
   bool bRetVal(false);
   switch (m_ConfigOption) {
     case CF_REGISTRY:
@@ -550,7 +699,7 @@ bool PWSprefs::DeletePref(const StringX &name)
       break;
     case CF_FILE_RW:
       bRetVal = (m_pXML_Config->DeleteSetting(m_csHKCU_PREF,
-                                             name.c_str()) == TRUE);
+                                              name.c_str()) == TRUE);
       break;
     case CF_FILE_RW_NEW:
     case CF_FILE_RO:
@@ -609,7 +758,7 @@ struct shortcut_less {
 bool equal_shortcuts(st_prefShortcut a, st_prefShortcut b)
 {
   return (a.id        == b.id &&
-          a.cVirtKey  == b.cVirtKey &&
+          a.siVirtKey == b.siVirtKey &&
           a.cModifier == b.cModifier);
 }
 
@@ -618,16 +767,17 @@ void PWSprefs::SetPrefShortcuts(const std::vector<st_prefShortcut> &vShortcuts)
   std::vector<st_prefShortcut> vSortedShortcuts(vShortcuts);
   std::sort(vSortedShortcuts.begin(), vSortedShortcuts.end(), shortcut_less());
   if (m_vShortcuts.size() == vSortedShortcuts.size() &&
-      std::equal(m_vShortcuts.begin(), m_vShortcuts.end(), 
+      std::equal(m_vShortcuts.begin(), m_vShortcuts.end(),
                  vSortedShortcuts.begin(), equal_shortcuts))
     return;
- 
+
   m_vShortcuts = vSortedShortcuts;
   m_prefs_changed[SHC_PREF] = true;
 }
 
 StringX PWSprefs::Store(bool bUseCopy)
 {
+  PWS_LOGIT_ARGS("bUseCopy=%s", bUseCopy ? _T("true") : _T("false"));
   /*
   * Create a string of values that are (1) different from the defaults, &&
   * (2) are storage in the database (ptype == ptDatabase)
@@ -650,18 +800,17 @@ StringX PWSprefs::Store(bool bUseCopy)
     p_intValues = m_intValues;
     p_stringValues = m_stringValues;
   }
- 
-  oStringXStream os;
-  int i;
 
-  for (i = 0; i < NumBoolPrefs; i++, p_boolValues++) {
+  oStringXStream os;
+
+  for (int i = 0; i < NumBoolPrefs; i++, p_boolValues++) {
     if (*p_boolValues != m_bool_prefs[i].defVal &&
         m_bool_prefs[i].ptype == ptDatabase) {
       os << _T("B ") << i << TCHAR(' ') << (*p_boolValues ? 1 : 0) << TCHAR(' ');
     }
   }
 
-  for (i = 0; i < NumIntPrefs; i++, p_intValues++) {
+  for (int i = 0; i < NumIntPrefs; i++, p_intValues++) {
     if (*p_intValues != m_int_prefs[i].defVal &&
         m_int_prefs[i].ptype == ptDatabase) {
       os << _T("I ") << i << TCHAR(' ') << *p_intValues << TCHAR(' ');
@@ -672,30 +821,53 @@ StringX PWSprefs::Store(bool bUseCopy)
   const TCHAR Delimiters[] = _T("\"\'#?!%&*+=:;@~<>?,.{}[]()\xbb");
   const int NumDelimiters = sizeof(Delimiters) / sizeof(Delimiters[0]) - 1;
 
-  for (i = 0; i < NumStringPrefs; i++, p_stringValues++) {
+  for (int i = 0; i < NumStringPrefs; i++, p_stringValues++) {
     if (*p_stringValues != m_string_prefs[i].defVal &&
         m_string_prefs[i].ptype == ptDatabase) {
       const StringX svalue = *p_stringValues;
-      delim = _T(' ');
+      delim = TCHAR(' ');
       for (int j = 0; j < NumDelimiters; j++) {
         if (svalue.find(Delimiters[j]) == StringX::npos) {
           delim = Delimiters[j];
           break;
         }
       }
-      if (delim == _T(' '))
+      if (delim == TCHAR(' '))
         continue;  // We tried, but just can't save it!
 
-      os << _T("S ") << i << _T(' ') << delim << *p_stringValues <<
-        delim << _T(' ');
+      os << _T("S ") << i << TCHAR(' ') << delim << *p_stringValues <<
+        delim << TCHAR(' ');
     }
   }
 
+  // Now add preferences not known to this release (current types B, I & S)
+  for (size_t i = 0; i < m_vUnknownBPrefs.size(); i++) {
+    //
+    os << _T("B ") << m_vUnknownBPrefs[i].index << TCHAR(' ') <<
+        (m_vUnknownBPrefs[i].bValue ? 1 : 0) << TCHAR(' ');
+  }
+
+  for (size_t i = 0; i < m_vUnknownIPrefs.size(); i++) {
+    //
+    os << _T("I ") << m_vUnknownIPrefs[i].index << TCHAR(' ') <<
+        m_vUnknownIPrefs[i].iValue << TCHAR(' ');
+  }
+
+  for (size_t i = 0; i < m_vUnknownSPrefs.size(); i++) {
+    //
+    os << _T("S ") << m_vUnknownSPrefs[i].index << TCHAR(' ') <<
+        m_vUnknownSPrefs[i].delim <<
+        m_vUnknownSPrefs[i].sValue.c_str() <<
+        m_vUnknownSPrefs[i].delim << TCHAR(' ');
+  }
+
   return os.str();
 }
 
 void PWSprefs::Load(const StringX &prefString, bool bUseCopy)
 {
+  PWS_LOGIT_ARGS("bUseCopy=%s", bUseCopy ? _T("true") : _T("false"));
+
   bool *p_boolValues;
   unsigned int *p_intValues;
   StringX *p_stringValues;
@@ -731,6 +903,8 @@ void PWSprefs::Load(const StringX &prefString, bool bUseCopy)
     }
   }
 
+  ClearUnknownPrefs();
+
   if (prefString.empty())
     return;
 
@@ -748,22 +922,38 @@ void PWSprefs::Load(const StringX &prefString, bool bUseCopy)
     is >> type >> index;
     if (is.eof())
       break;
+
     switch (type) {
       case TCHAR('B'):
         // Need to get value - even if not understood or wanted
         is >> ival;
         // forward compatibility and check whether still in DB
-        if (index < NumBoolPrefs && m_bool_prefs[index].ptype == ptDatabase) {
+        if (index < NumBoolPrefs) {
+          if (m_bool_prefs[index].ptype == ptDatabase) {
+            ASSERT(ival == 0 || ival == 1);
+            p_boolValues[index] = (ival != 0);
+          }
+        } else {
+          st_BP stxBP;
           ASSERT(ival == 0 || ival == 1);
-          p_boolValues[index] = (ival != 0);
+          stxBP.index = index;
+          stxBP.bValue = ival == 1;
+          m_vUnknownBPrefs.push_back(stxBP);
         }
         break;
       case TCHAR('I'):
         // Need to get value - even if not understood or wanted
         is >> iuval;
         // forward compatibility and check whether still in DB
-        if (index < NumIntPrefs && m_int_prefs[index].ptype == ptDatabase) {
-          p_intValues[index] = iuval;
+        if (index < NumIntPrefs) {
+          if (m_int_prefs[index].ptype == ptDatabase) {
+            p_intValues[index] = iuval;
+          }
+        } else {
+          st_IP stxIP;
+          stxIP.index = index;
+          stxIP.iValue = iuval;
+          m_vUnknownIPrefs.push_back(stxIP);
         }
         break;
       case TCHAR('S'):
@@ -773,8 +963,16 @@ void PWSprefs::Load(const StringX &prefString, bool bUseCopy)
         is.get(buf, N, delim[0]); // get string value
         is.ignore(1, TCHAR(' ')); // skip over trailing delimiter
         // forward compatibility and check whether still in DB
-        if (index < NumStringPrefs && m_string_prefs[index].ptype == ptDatabase) {
-          p_stringValues[index] = buf;
+        if (index < NumStringPrefs) {
+          if (m_string_prefs[index].ptype == ptDatabase) {
+            p_stringValues[index] = buf;
+          }
+        } else {
+          st_SP stxSP;
+          stxSP.index = index;
+          stxSP.delim = delim[0];
+          stxSP.sValue = buf;
+          m_vUnknownSPrefs.push_back(stxSP);
         }
         break;
       default:
@@ -806,7 +1004,7 @@ void PWSprefs::UpdateTimeStamp()
   if (m_ConfigOption == CF_FILE_RW || m_ConfigOption == CF_FILE_RW_NEW) {
     time_t time_now;
     time(&time_now);
-    const StringX now = PWSUtil::ConvertToDateTimeString(time_now, TMC_XML);
+    const StringX now = PWSUtil::ConvertToDateTimeString(time_now, PWSUtil::TMC_XML);
 
     m_pXML_Config->Set(m_csHKCU, _T("LastUpdated"), now.c_str());
   }
@@ -838,11 +1036,10 @@ void PWSprefs::FindConfigFile()
    */
 
   const stringT sExecDir = PWSdirs::GetExeDir();
-  const stringT sCnfgDir = PWSdirs::GetConfigDir();
-  PWSdirs dirs(sCnfgDir);
 
   // Set path & name of config file
   if (!m_userSetCfgFile) { // common case
+    const stringT sCnfgDir = PWSdirs::GetConfigDir();
     m_configfilename = sExecDir + cfgFileName;
     if (pws_os::FileExists(m_configfilename)) {
       // old (exe dir) exists, is host/user there?
@@ -864,6 +1061,8 @@ void PWSprefs::FindConfigFile()
 
 void PWSprefs::InitializePreferences()
 {
+  PWS_LOGIT;
+
   // Set up XML "keys": host/user ensure that they start with letter,
   // and otherwise conforms with http://www.w3.org/TR/2000/REC-xml-20001006#NT-Name
   const SysInfo *si = SysInfo::GetInstance();
@@ -871,9 +1070,11 @@ void PWSprefs::InitializePreferences()
   XMLify(charT('H'), hn);
   stringT un = si->GetEffectiveUser();
   XMLify(charT('u'), un);
-  m_csHKCU = hn.c_str();
+  m_csHKCU = _T("Pwsafe_Settings\\");
+  m_csHKCU += hn.c_str();
   m_csHKCU += _T("\\");
   m_csHKCU += un.c_str();
+
   // set up other keys
   m_csHKCU_MRU  = m_csHKCU + _T("\\MRU");
   m_csHKCU_POS  = m_csHKCU + _T("\\Position");
@@ -905,17 +1106,9 @@ void PWSprefs::InitializePreferences()
   } else {
     // Doesn't exist but can we write to the directory?
     // Try and create the file (and delete afterwards if we succeeded)
-#ifdef UNICODE
-    CUTF8Conv conv;
-    size_t fnamelen;
-    const unsigned char *fname = NULL;
-    conv.ToUTF8(m_configfilename.c_str(), fname, fnamelen); 
-#else
-    const char *fname = m_configfilename.c_str();
-#endif
-    ofstream ofs(reinterpret_cast<const char *>(fname));
-    if (!ofs.bad()) {
-      ofs.close();
+    FILE *testfile = pws_os::FOpen(m_configfilename.c_str(), _T("w"));
+    if (testfile != NULL) {
+      fclose(testfile);
       pws_os::DeleteAFile(m_configfilename.c_str());
       m_ConfigOption = CF_FILE_RW_NEW;
       isRO = false;
@@ -991,33 +1184,38 @@ void PWSprefs::InitializePreferences()
 
 void PWSprefs::SetDatabasePrefsToDefaults(const bool bUseCopy)
 {
+  PWS_LOGIT_ARGS("bUseCopy=%s", bUseCopy ? _T("true") : _T("false"));
+
   // Set Database prefs to hardcoded values
   int i;
   // Default values only
   for (i = 0; i < NumBoolPrefs; i++)
-    if (m_bool_prefs[i].ptype == ptDatabase)
+    if (m_bool_prefs[i].ptype == ptDatabase) {
       if (bUseCopy)
         m_boolCopyValues[i] = m_bool_prefs[i].defVal != 0;
       else
         m_boolValues[i] = m_bool_prefs[i].defVal != 0;
-
+    }
   for (i = 0; i < NumIntPrefs; i++)
-    if (m_int_prefs[i].ptype == ptDatabase)
+    if (m_int_prefs[i].ptype == ptDatabase) {
       if (bUseCopy)
         m_intCopyValues[i] = m_int_prefs[i].defVal;
       else
         m_intValues[i] = m_int_prefs[i].defVal;
-
+    }
   for (i = 0; i < NumStringPrefs; i++)
-    if (m_string_prefs[i].ptype == ptDatabase)
+    if (m_string_prefs[i].ptype == ptDatabase) {
       if (bUseCopy)
         m_stringCopyValues[i] = m_string_prefs[i].defVal;
       else
         m_stringValues[i] = m_string_prefs[i].defVal;
+    }
 }
 
 void PWSprefs::LoadProfileFromDefaults()
 {
+  PWS_LOGIT;
+
   // set prefs to hardcoded values
   int i;
   // Default values only
@@ -1036,6 +1234,8 @@ void PWSprefs::LoadProfileFromDefaults()
 
 void PWSprefs::LoadProfileFromRegistry()
 {
+  PWS_LOGIT;
+
   // Read in values from registry
   if (!m_bRegistryKeyExists)
     return; // Avoid creating keys if none already, as
@@ -1062,14 +1262,14 @@ void PWSprefs::LoadProfileFromRegistry()
     // to correct spelling while maintain preference's value.
     bool bccom = GetPref(ClearClipboardOnMinimize);
     bool bccoe = GetPref(ClearClipboardOnExit);
-    
+
     bool bccom2 = pws_os::RegReadValue(PWS_REG_OPTIONS,
                                        _T("ClearClipoardOnMinimize"), // deliberate!
                                        bccom) != 0;
     bool bccoe2 = pws_os::RegReadValue(PWS_REG_OPTIONS,
                                        _T("ClearClipoardOneExit"), // deliberate!
                                        bccoe) != 0;
-    
+
     // If old (mis-spelt) name was there, use its value. Since the
     // default above was the new (correct) spelling, it has priority
     m_boolValues[ClearClipboardOnMinimize] = bccom2;
@@ -1140,6 +1340,8 @@ void PWSprefs::LoadProfileFromRegistry()
 
 bool PWSprefs::LoadProfileFromFile()
 {
+  PWS_LOGIT;
+
   /*
   * Called from InitializePreferences() at startup,
   * attempts to read in application preferences
@@ -1169,7 +1371,7 @@ bool PWSprefs::LoadProfileFromFile()
     goto exit;
   }
 
-  // LockOnIdleTimeout is now a Database preference - 
+  // LockOnIdleTimeout is now a Database preference -
   // Silently delete it from XML file (actually doesn't delete
   // it here but marks as changed so will be deleted when saved).
   // Set updated so that the XML file is rewritten without it
@@ -1266,15 +1468,12 @@ exit:
 
 void PWSprefs::SaveApplicationPreferences()
 {
+  PWS_LOGIT;
+
   int i;
   if (!m_prefs_changed[APP_PREF])
     return;
 
-  // change to config dir
-  // dirs' d'tor will put us back when we leave
-  // needed for case where m_configfilename was passed relatively
-  PWSdirs dirs(PWSdirs::GetConfigDir());
-
   if (m_ConfigOption == CF_FILE_RW ||
       m_ConfigOption == CF_FILE_RW_NEW) {
     // Load prefs file in case it was changed elsewhere
@@ -1410,16 +1609,19 @@ void PWSprefs::SaveApplicationPreferences()
       m_ConfigOption == CF_FILE_RW_NEW) {
     int j;
     const int n = GetPref(PWSprefs::MaxMRUItems);
-    // Delete ALL entries
+    // Delete ALL MRU entries
     m_pXML_Config->DeleteSetting(m_csHKCU_MRU, _T(""));
     // Now put back the ones we want
     stringT csSubkey;
     for (j = 0; j < n; j++) {
       if (!m_MRUitems[j].empty()) {
-        Format(csSubkey, _T("Safe%02d"), j+1);
+        Format(csSubkey, _T("Safe%02d"), j + 1);
         m_pXML_Config->Set(m_csHKCU_MRU, csSubkey, m_MRUitems[j]);
       }
     }
+
+    // Since we may have changed the MRU - update timestamp in config file
+    UpdateTimeStamp();
   }
 
   if (m_ConfigOption == CF_FILE_RW ||
@@ -1444,11 +1646,6 @@ void PWSprefs::SaveShortcuts()
   if (!m_prefs_changed[SHC_PREF])
     return;
 
-  // change to config dir
-  // dirs' d'tor will put us back when we leave
-  // needed for case where m_configfilename was passed relatively
-  PWSdirs dirs(PWSdirs::GetConfigDir());
-
   if (m_ConfigOption == CF_FILE_RW ||
       m_ConfigOption == CF_FILE_RW_NEW) {
     // Load prefs file in case it was changed elsewhere
@@ -1475,6 +1672,9 @@ void PWSprefs::SaveShortcuts()
     // Now put back the ones we want
     if (!m_vShortcuts.empty())
       m_pXML_Config->SetShortcuts(m_csHKCU_SHCT, m_vShortcuts);
+
+    // Since we may have changed the shortcuts - update timestamp in config file
+    UpdateTimeStamp();
   }
 
   if (m_ConfigOption == CF_FILE_RW ||
@@ -1516,7 +1716,7 @@ int PWSprefs::GetConfigIndicator() const
     case CF_FILE_RW:
     case CF_FILE_RW_NEW:
       return IDSC_CONFIG_FILE_RW;
-    case CF_FILE_RO: 
+    case CF_FILE_RO:
       return IDSC_CONFIG_FILE_RO;
     default:
       ASSERT(0);
@@ -1533,6 +1733,7 @@ void PWSprefs::ImportOldPrefs()
 {
   if (!pws_os::RegOpenSubtree(OldSubKey.c_str()))
     return;
+
   // Iterate over app preferences (those not stored
   // in database), read values and store if found.
   int i;
@@ -1591,6 +1792,8 @@ void PWSprefs::DeleteOldPrefs()
 
 stringT PWSprefs::GetXMLPreferences()
 {
+  PWS_LOGIT;
+
   stringT retval(_T(""));
   ostringstreamT os;
 
@@ -1599,7 +1802,7 @@ stringT PWSprefs::GetXMLPreferences()
   for (i = 0; i < NumBoolPrefs; i++) {
     if (m_boolValues[i] != m_bool_prefs[i].defVal &&
         m_bool_prefs[i].ptype == ptDatabase)
-      os << "\t\t<" << m_bool_prefs[i].name << ">" << (m_boolValues[i] ? "1</" : "0</") << 
+      os << "\t\t<" << m_bool_prefs[i].name << ">" << (m_boolValues[i] ? "1</" : "0</") <<
       m_bool_prefs[i].name << ">" << endl;
   }
 
@@ -1630,37 +1833,9 @@ stringT PWSprefs::GetXMLPreferences()
   for (i = 0; i < NumStringPrefs; i++) {
     if (m_stringValues[i] != m_string_prefs[i].defVal &&
         m_string_prefs[i].ptype == ptDatabase) {
-      StringX::size_type p = m_stringValues[i].find(_T("]]>")); // special handling required
-      if (p == StringX::npos) {
-        // common case
-        os << "\t\t<" << m_string_prefs[i].name << "><![CDATA[" <<
-          m_stringValues[i] << "]]></" << 
+      stringT sTemp = PWSUtil::GetSafeXMLString(m_stringValues[i]);
+      os << "\t\t<" << m_string_prefs[i].name << ">" << sTemp << "</" <<
           m_string_prefs[i].name << ">" << endl;
-      } else {
-        // value has "]]>" sequence(s) that need(s) to be escaped
-        // Each "]]>" splits the field into two CDATA sections, one ending with
-        // ']]', the other starting with '>'
-        const StringX value = m_stringValues[i];
-        os << "\t\t<" << m_string_prefs[i].name << ">";
-        size_t from = 0, to = p + 2;
-        do {
-          StringX slice = value.substr(from, (to - from));
-          os << "<![CDATA[" << slice << "]]><![CDATA[";
-          from = to;
-          p = value.find(_T("]]>"), from); // are there more?
-          if (p == StringX::npos) {
-            to = value.length();
-            slice = value.substr(from, (to - from));
-          } else {
-            to = p + 2;
-            slice = value.substr(from, (to - from));
-            from = to;
-            to = value.length();
-          }
-          os <<  slice << "]]>";
-        } while (p != StringX::npos);
-        os << "</" << m_string_prefs[i].name << ">" << endl;      
-      }
     }
   }
   os << "\t</Preferences>" << endl << endl;
@@ -1670,7 +1845,7 @@ stringT PWSprefs::GetXMLPreferences()
 
 bool PWSprefs::LockCFGFile(const stringT &filename, stringT &locker)
 {
-  return pws_os::LockFile(filename, locker, 
+  return pws_os::LockFile(filename, locker,
                           s_cfglockFileHandle, s_cfgLockCount);
 }
 
@@ -1684,3 +1859,10 @@ bool PWSprefs::IsLockedCFGFile(const stringT &filename)
 {
   return pws_os::IsLockedFile(filename);
 }
+
+void PWSprefs::ClearUnknownPrefs()
+{
+  m_vUnknownBPrefs.clear();
+  m_vUnknownIPrefs.clear();
+  m_vUnknownSPrefs.clear();
+}
