@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -69,11 +69,6 @@
 #define LTC_NO_ROLC
 #endif
 
-// stop MS VC++ 6 and earlier being quite so pedantic
-#if defined(_MSC_VER) && (_MSC_VER <= 1200) // i.e. only MSVC++ 6 and earlier
-#pragma warning ( disable : 4097 )
-#endif
-
 #undef PWS_PLATFORM
 #undef POCKET_PC
 
