@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -44,8 +44,7 @@ MFileXMLProcessor::~MFileXMLProcessor()
 // ---------------------------------------------------------------------------
 bool MFileXMLProcessor::Process(const bool &bvalidation, const stringT &ImportedPrefix,
                                 const stringT &strXMLFileName, const stringT &strXSDFileName,
-                                const bool &bImportPSWDsOnly,
-                                int &nITER)
+                                const bool &bImportPSWDsOnly)
 {
   HRESULT hr, hr0, hr60;
   bool b_ok = false;
@@ -137,11 +136,7 @@ bool MFileXMLProcessor::Process(const bool &bvalidation, const stringT &Imported
     //  Let's begin the parsing now
     wchar_t wcURL[MAX_PATH] = {0};
 #ifdef _UNICODE
-#if (_MSC_VER >= 1400)
     _tcscpy_s(wcURL, MAX_PATH, strXMLFileName.c_str());
-#else
-    _tcscpy(wcURL, strXMLFileName.c_str());
-#endif
 #else
     mbstowcs(wcURL, strXMLFileName.c_str(), strXMLFileName.length());
 #endif
@@ -165,6 +160,7 @@ bool MFileXMLProcessor::Process(const bool &bvalidation, const stringT &Imported
           m_numEntriesPWHErrors = pCH->getNumPWHErrors();
           m_numNoPolicies = pCH->getNumNoPolicies();
           m_numRenamedPolicies = pCH->getNumRenamedPolicies();
+          m_numShortcutsRemoved = pCH->getNumShortcutsRemoved();
 
           // Get lists
           m_strXMLErrors = pCH->getXMLErrors();
@@ -173,7 +169,6 @@ bool MFileXMLProcessor::Process(const bool &bvalidation, const stringT &Imported
           m_strRenameList = pCH->getRenameList();
 
           if (b_into_empty) {
-            nITER = pCH->m_nITER;
             pCH->AddDBPreferences();
           }
         }
