@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -48,8 +48,6 @@ public:
   size_t m_pwsymbolminlength;
   size_t m_pwupperminlength;
 
-  int m_useownsymbols;
-
   //}}AFX_DATA
 
   // Overrides
@@ -57,17 +55,16 @@ public:
   //{{AFX_VIRTUAL(CAddEdit_PasswordPolicy)
 protected:
   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
-  virtual BOOL OnApply();
   BOOL PreTranslateMessage(MSG* pMsg);
+  virtual BOOL OnApply();
+  virtual BOOL OnKillActive();
   //}}AFX_VIRTUAL
 
   // Implementation
-protected:
   // Generated message map functions
   //{{AFX_MSG(CAddEdit_PasswordPolicy)
   virtual BOOL OnInitDialog();
   afx_msg void OnHelp();
-  afx_msg BOOL OnKillActive();
   afx_msg LRESULT OnQuerySiblings(WPARAM wParam, LPARAM );
 
   afx_msg void OnChanged();
@@ -81,7 +78,6 @@ protected:
   afx_msg void OnMakePronounceable();
   afx_msg void OnSelectNamedPolicy();
   afx_msg void OnSetSpecificPWPolicy();
-  afx_msg void OnSymbols();
   afx_msg void OnOwnSymbolsChanged();
   afx_msg void OnNamesComboChanged();
   //}}AFX_MSG
@@ -97,7 +93,8 @@ private:
 
   void do_hex(const bool bHex);                 // bHex == true enable hex
   void do_easyorpronounceable(const bool bSet); // bSet == true enable one of these options
-
+  enum UseX {USELOWER = 0, USEUPPER = 1, USEDIGITS = 2, USESYM = 3};
+  void do_useX(UseX x, size_t &minlength); // used by OnUse{LowerCase,UpperCase,Digits,Symbols}
   // number of checkboxes & lengths disabled when hex chosen
   enum {N_NOHEX = 6, N_HEX_LENGTHS = 4};
   static const UINT nonHex[N_NOHEX];                  // IDs of said checkboxes
@@ -111,6 +108,8 @@ private:
   bool m_bInitdone;
 
   int m_policy_radibtn;  // Can't use M_policy() anymore
+public:
+    afx_msg void OnSymbolReset();
 };
 //-----------------------------------------------------------------------------
 // Local variables:
