@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -10,10 +10,15 @@
 
 #include "stdafx.h"
 #include "ControlExtns.h"
+
 #include "core/ItemField.h" // for CSecEditExtn
 #include "core/BlowFish.h"  // ditto
 #include "core/PWSrand.h"   // ditto
-#include "resource2.h" // for CEditExtn context menu
+#include "core/PWCharPool.h" // for CSymbolEdit
+#include "resource2.h"     // for CEditExtn context menu
+
+#include "vsstyle.h"
+
 #include <algorithm>
 
 #ifdef _DEBUG
@@ -281,6 +286,8 @@ HBRUSH CEditExtn::CtlColor(CDC* pDC, UINT /*nCtlColor*/)
   }
 }
 
+// Following structure used by both CEditExtn and CRichEditExtn for
+// finding menu items in the supplied extra vector based on command message number
 struct equal_cmd {
   equal_cmd(UINT_PTR const& nCmd) : m_nCmd(nCmd) {}
   bool operator()(st_context_menu const& context_menu) const
@@ -343,6 +350,8 @@ void CEditExtn::OnContextMenu(CWnd* pWnd, CPoint point)
       menu.InsertMenu((int)i + iPos, MF_BYPOSITION | m_vmenu_items[i].flags,
                       m_vmenu_items[i].message_number,
                       m_vmenu_items[i].menu_string.c_str());
+      if (!m_vmenu_items[i].bEnable)
+        menu.EnableMenuItem((int)i + iPos, MF_BYPOSITION | MF_GRAYED);
   }
 
   if (point.x == -1 || point.y == -1) {
@@ -363,7 +372,7 @@ void CEditExtn::OnContextMenu(CWnd* pWnd, CPoint point)
   iter = std::find_if(m_vmenu_items.begin(), m_vmenu_items.end(),
                       equal_cmd(nCmd));
   if (iter != m_vmenu_items.end()) {
-    this->GetParent()->SendMessage((UINT)nCmd);
+    this->GetParent()->SendMessage((UINT)nCmd, iter->wParam, iter->lParam);
     return;
   }
 
@@ -383,18 +392,266 @@ void CEditExtn::OnContextMenu(CWnd* pWnd, CPoint point)
   }
 }
 
-void CEditExtn::UpdateState(const int message_number, const BOOL new_state)
+void CEditExtn::UpdateState(const int message_number, const bool new_state)
 {
   std::vector<st_context_menu>::iterator iter;
   iter = std::find_if(m_vmenu_items.begin(), m_vmenu_items.end(),
                       equal_cmd(message_number));
   if (iter != m_vmenu_items.end()) {
-    int flags = new_state == TRUE ? MF_CHECKED : MF_UNCHECKED;
+    int flags = new_state ? MF_CHECKED : MF_UNCHECKED;
     iter->flags = flags;
     return;
   }
 }
 
+void CEditExtn::EnableMenuItem(const int message_number, const bool bEnable)
+{
+  std::vector<st_context_menu>::iterator iter;
+  iter = std::find_if(m_vmenu_items.begin(), m_vmenu_items.end(),
+                      equal_cmd(message_number));
+  if (iter != m_vmenu_items.end()) {
+    iter->bEnable = bEnable;
+    return;
+  }
+}
+
+/////////////////////////////////////////////////////////////////////////////
+// CRichEditExtn
+
+CRichEditExtn::CRichEditExtn(COLORREF focusColor)
+  : m_bIsFocused(FALSE), m_lastposition(-1), m_crefInFocus(focusColor),
+  m_bContextMenu(false)
+{
+  m_vmenu_items.clear();
+}
+
+
+void CRichEditExtn::SetContextMenu(const std::vector<st_context_menu> &vmenu_items)
+{
+  // Don't allow if menu string is empty.
+  if (vmenu_items.empty()) {
+    m_vmenu_items.clear();
+  } else {
+    m_vmenu_items = vmenu_items;
+  }
+
+  m_hCursor = LoadCursor(NULL, IDC_ARROW);
+}
+
+CRichEditExtn::~CRichEditExtn()
+{
+}
+
+BEGIN_MESSAGE_MAP(CRichEditExtn, CRichEditCtrl)
+  //{{AFX_MSG_MAP(CRichEditExtn)
+  ON_WM_SETFOCUS()
+  ON_WM_KILLFOCUS()
+  ON_WM_CONTEXTMENU()
+  ON_WM_LBUTTONDOWN()
+  ON_WM_SETCURSOR()
+  //}}AFX_MSG_MAP
+END_MESSAGE_MAP()
+
+/////////////////////////////////////////////////////////////////////////////
+// CRichEditExtn message handlers
+
+void CRichEditExtn::GetSel(long &nStartChar, long &nEndChar)
+{
+  CRichEditCtrl::GetSel(nStartChar, nEndChar);
+  m_nStartChar = nStartChar;
+  m_nEndChar = nEndChar;
+}
+
+void CRichEditExtn::SetSel(long nStartChar, long nEndChar)
+{
+  m_nStartChar = nStartChar;
+  m_nEndChar = nEndChar;
+  CRichEditCtrl::SetSel(nStartChar, nEndChar);
+}
+
+void CRichEditExtn::OnSetFocus(CWnd* pOldWnd)
+{
+  m_bIsFocused = TRUE;
+  CRichEditCtrl::OnSetFocus(pOldWnd);
+  if (m_lastposition >= 0) {
+    int iLine = LineFromChar(m_lastposition);
+    LineScroll(iLine);
+    SetSel(m_nStartChar, m_nEndChar); 
+  }
+
+  SetBackgroundColor(FALSE, m_crefInFocus);
+  Invalidate(TRUE);
+}
+
+void CRichEditExtn::OnKillFocus(CWnd* pNewWnd)
+{
+  m_bIsFocused = FALSE;
+  m_lastposition = LineIndex();
+  GetSel(m_nStartChar, m_nEndChar);
+  CRichEditCtrl::OnKillFocus(pNewWnd);
+
+  SetBackgroundColor(FALSE, crefNoFocus);
+  Invalidate(TRUE);
+}
+
+void CRichEditExtn::OnLButtonDown(UINT nFlags, CPoint point)
+{
+  // Get the scroll bar position.
+  int nScrollHPos = GetScrollPos(SB_HORZ);
+  int nScrollVPos = GetScrollPos(SB_VERT);
+
+  int n = CharFromPos(point);
+  m_lastposition = HIWORD(n);
+  m_nStartChar = m_nEndChar = LOWORD(n);
+
+  CRichEditCtrl::OnLButtonDown(nFlags, point);
+
+  // Reset the scroll bar position.
+  SetScrollPos(SB_HORZ, nScrollHPos);
+  SetScrollPos(SB_VERT, nScrollVPos);
+
+  // Reset the display scroll position.
+  SendMessage(WM_HSCROLL, MAKEWPARAM(SB_THUMBTRACK, nScrollHPos), 0);
+  SendMessage(WM_VSCROLL, MAKEWPARAM(SB_THUMBTRACK, nScrollVPos), 0);
+
+  SetSel(m_nStartChar, m_nEndChar);
+}
+
+BOOL CRichEditExtn::OnSetCursor(CWnd *pWnd, UINT nHitTest, UINT message)
+{
+  if (m_bContextMenu) {
+    ::SetCursor(m_hCursor);
+    return TRUE;
+  }
+  return CRichEditCtrl::OnSetCursor(pWnd, nHitTest, message);
+}
+
+void CRichEditExtn::OnContextMenu(CWnd* pWnd, CPoint point)
+{
+  if (m_vmenu_items.empty()) {
+    CRichEditExtn::OnContextMenu(pWnd, point);
+    return;
+  }
+
+  SetFocus();
+  CMenu menu;
+  menu.CreatePopupMenu();
+
+  BOOL bReadOnly = GetStyle() & ES_READONLY;
+  DWORD flags = CanUndo() && !bReadOnly ? 0 : MF_GRAYED;
+
+  int iPos(0);
+  menu.InsertMenu(iPos++, MF_BYPOSITION | flags, EM_UNDO,
+                  CString(MAKEINTRESOURCE(IDS_MENUSTRING_UNDO)));
+
+  menu.InsertMenu(iPos++, MF_BYPOSITION | MF_SEPARATOR);
+
+  long lStart, lEnd;
+  GetSel(lStart, lEnd);
+  flags = lStart == lEnd ? MF_GRAYED : 0;
+  menu.InsertMenu(iPos++, MF_BYPOSITION | flags, WM_COPY,
+                  CString(MAKEINTRESOURCE(IDS_MENUSTRING_COPY)));
+
+  flags = (flags == MF_GRAYED || bReadOnly) ? MF_GRAYED : 0;
+  menu.InsertMenu(iPos++, MF_BYPOSITION | flags, WM_CUT,
+                  CString(MAKEINTRESOURCE(IDS_MENUSTRING_CUT)));
+
+  flags = (flags == MF_GRAYED || bReadOnly) ? MF_GRAYED : 0;
+  menu.InsertMenu(iPos++, MF_BYPOSITION | flags, WM_CLEAR,
+                  CString(MAKEINTRESOURCE(IDS_MENUSTRING_DELETE)));
+
+  flags = IsClipboardFormatAvailable(EDIT_CLIPBOARD_TEXT_FORMAT) &&
+                                     !bReadOnly ? 0 : MF_GRAYED;
+  menu.InsertMenu(iPos++, MF_BYPOSITION | flags, WM_PASTE,
+                  CString(MAKEINTRESOURCE(IDS_MENUSTRING_PASTE)));
+
+  menu.InsertMenu(iPos++, MF_BYPOSITION | MF_SEPARATOR);
+
+  int len = GetWindowTextLength();
+  flags = (!len || (lStart == 0 && lEnd == len)) ? MF_GRAYED : 0;
+
+  menu.InsertMenu(iPos++, MF_BYPOSITION | flags, EM_SELECTALL,
+                  CString(MAKEINTRESOURCE(IDS_MENUSTRING_SELECTALL)));
+
+  menu.InsertMenu(iPos++, MF_BYPOSITION | MF_SEPARATOR);
+
+  for (size_t i = 0; i < m_vmenu_items.size(); i++) {
+      menu.InsertMenu((int)i + iPos, MF_BYPOSITION | m_vmenu_items[i].flags,
+                      m_vmenu_items[i].message_number,
+                      m_vmenu_items[i].menu_string.c_str());
+      if (!m_vmenu_items[i].bEnable)
+        menu.EnableMenuItem((int)i + iPos, MF_BYPOSITION | MF_GRAYED);
+  }
+
+  if (point.x == -1 || point.y == -1) {
+    CRect rc;
+    GetClientRect(&rc);
+    point = rc.CenterPoint();
+    ClientToScreen(&point);
+  }
+
+  // For some reason - weird cursor
+  m_bContextMenu = true;
+  
+  // Now show menu
+  UINT_PTR nCmd = menu.TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON |
+                                      TPM_RETURNCMD | TPM_RIGHTBUTTON, 
+                                      point.x, point.y, this);
+  // Restore cursor
+  m_bContextMenu = false;
+
+  if (nCmd == 0)
+    return;
+
+  std::vector<st_context_menu>::iterator iter;
+  iter = std::find_if(m_vmenu_items.begin(), m_vmenu_items.end(),
+                      equal_cmd(nCmd));
+  if (iter != m_vmenu_items.end()) {
+    this->GetParent()->SendMessage((UINT)nCmd, iter->wParam, iter->lParam);
+    return;
+  }
+
+  switch (nCmd) {
+    case EM_UNDO:
+    case WM_CUT:
+    case WM_COPY:
+    case WM_CLEAR:
+      SendMessage((UINT)nCmd);
+      break;
+    case WM_PASTE:
+      SendMessage(EM_PASTESPECIAL, CF_UNICODETEXT, NULL);
+      break;
+    case EM_SELECTALL:
+      SendMessage(EM_SETSEL, 0, -1);
+      break;
+    default:
+      break;
+  }
+}
+
+void CRichEditExtn::UpdateState(const int message_number, const bool new_state)
+{
+  std::vector<st_context_menu>::iterator iter;
+  iter = std::find_if(m_vmenu_items.begin(), m_vmenu_items.end(),
+                      equal_cmd(message_number));
+  if (iter != m_vmenu_items.end()) {
+    int flags = new_state ? MF_CHECKED : MF_UNCHECKED;
+    iter->flags = flags;
+    return;
+  }
+}
+
+void CRichEditExtn::EnableMenuItem(const int message_number, const bool bEnable)
+{
+  std::vector<st_context_menu>::iterator iter;
+  iter = std::find_if(m_vmenu_items.begin(), m_vmenu_items.end(),
+                      equal_cmd(message_number));
+  if (iter != m_vmenu_items.end()) {
+    iter->bEnable = bEnable;
+    return;
+  }
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // CListBoxExtn
 
@@ -606,8 +863,10 @@ END_MESSAGE_MAP()
 /////////////////////////////////////////////////////////////////////////////
 // CComboBoxExt message handlers
 
-HBRUSH CComboBoxExtn::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
+HBRUSH CComboBoxExtn::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
 {
+  HBRUSH hbr = CComboBox::OnCtlColor(pDC, pWnd, nCtlColor);
+
   if (nCtlColor == CTLCOLOR_EDIT) {
     // Extended Edit control
     if (m_edit.GetSafeHwnd() == NULL)
@@ -624,7 +883,7 @@ HBRUSH CComboBoxExtn::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
     }
   }
 
-  return CComboBox::OnCtlColor(pDC, pWnd, nCtlColor);
+  return hbr;
 }
 
 void CComboBoxExtn::OnDestroy()
@@ -668,7 +927,14 @@ struct CSecEditExtn::Impl {
     SecureZeroMemory(key, sizeof(key));
   }
 
-  ~Impl() {delete m_bf;}
+  ~Impl() {
+    delete m_bf;
+    // Following to clear the keyboard buffer
+    BYTE bytKeyBoardState[256] = {0};
+    
+    BOOL brc = ::SetKeyboardState(bytKeyBoardState);
+    ASSERT(brc);
+  }
   CItemField m_field;
   BlowFish *m_bf;
 };
@@ -829,6 +1095,31 @@ BEGIN_MESSAGE_MAP(CSymbolEdit, CEdit)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
+CSymbolEdit::CSymbolEdit() : CEdit(), m_validSym(CPasswordCharPool::GetDefaultSymbols())
+{
+}
+
+void CSymbolEdit::SetValidSym(const stringT &s)
+{
+  // Set the member variable.
+  // it's the caller's responsibility to decide
+  // whether or not to filter current value of
+  // the control to new set.
+  m_validSym = s;
+
+# if 0
+  // Here's one way to filter text to respect
+  // new symbol set
+  CString curSyms, newSyms;
+  GetWindowText(curSyms);
+  size_t slen = m_validSym.length();
+  for (size_t i = 0; i < slen; i++)
+    if (curSyms.Find(m_validSym[i]))
+      newSyms += m_validSym[i];
+  SetWindowText(newSyms);
+#endif
+}
+
 void CSymbolEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
 {
   //Allowed inputs are passed on to the base class
@@ -838,8 +1129,8 @@ void CSymbolEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
   }
 
   wint_t wChar = reinterpret_cast<wint_t &>(nChar);
-  // Must not be alphanumeric nor have and Alt/Ctrl key
-  if ((_istalpha(wChar) == 0 && _istdigit(wChar) == 0 && (nFlags & 0xE0800000) == 0)) {
+  // Must be a valid symbol, per current valid set
+  if (m_validSym.find(wChar) != std::string::npos) {
     CString cs_text;
     GetWindowText(cs_text);
     // Must not have duplicates
@@ -855,7 +1146,7 @@ LRESULT CSymbolEdit::OnPaste(WPARAM , LPARAM )
     return 0L; 
  
   std::wstring cs_data;
-	HANDLE hData = GetClipboardData(CF_UNICODETEXT);
+  HANDLE hData = GetClipboardData(CF_UNICODETEXT);
   if (hData != NULL) {
     wchar_t *buffer = (wchar_t *)GlobalLock(hData);
     if (buffer != NULL) {
@@ -863,16 +1154,16 @@ LRESULT CSymbolEdit::OnPaste(WPARAM , LPARAM )
       GlobalUnlock(hData);
     }
   }
-	CloseClipboard();
+  CloseClipboard();
 
   CString cs_text, cs_oldtext;;
   GetWindowText(cs_text);
   cs_oldtext = cs_text;
 
-  // Must not be alphanumeric
   for (size_t i = 0; i < cs_data.length(); i++) {
     wchar_t wChar = cs_data.at(i);
-    if ((_istalpha(wChar) == 0 && _istdigit(wChar) == 0)) {
+  // Must be a valid symbol, per current valid set
+    if (m_validSym.find(wChar) != std::string::npos) {
       // Must not have duplicates
       if (cs_text.Find(wChar) == -1)
         cs_text += wChar;
@@ -883,4 +1174,160 @@ LRESULT CSymbolEdit::OnPaste(WPARAM , LPARAM )
     SetWindowText(cs_text);
 
   return 0L;
-}
\ No newline at end of file
+}
+
+/////////////////////////////////////////////////////////////////////////////
+// CButtonExtn
+
+CButtonExtn::CButtonExtn()
+  : m_bUseTextColour(false), m_bUseBkgColour(false),
+  m_caption(L""), m_type(BS_AUTOCHECKBOX)
+{
+}
+
+CButtonExtn::~CButtonExtn()
+{
+}
+
+BEGIN_MESSAGE_MAP(CButtonExtn, CButton)
+  //{{AFX_MSG_MAP(CButtonExtn)
+  ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, OnCustomDraw)
+  //}}AFX_MSG_MAP
+END_MESSAGE_MAP()
+
+void CButtonExtn::SetType(int type)
+{
+  // Only the following are supported
+  if (type != BS_AUTOCHECKBOX && type != BS_AUTORADIOBUTTON) {
+    ASSERT(0);
+    return;
+  }
+
+  m_type = type;
+}
+
+void CButtonExtn::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
+{
+  // Code originally by Nikita Leontiev in answer to "Change checkBox text color Win32"
+  // in MS's Forum: "Visual Studio Developer Center > Visual Studio vNext Forums > Visual C++ General"
+  // Modified for MFC, Checkbox and Radio buttons by DK
+
+  LPNMCUSTOMDRAW lpNMCustomDraw = (LPNMCUSTOMDRAW)pNotifyStruct;
+  *pLResult = CDRF_DODEFAULT;
+
+  switch (lpNMCustomDraw->dwDrawStage) {
+    case CDDS_PREPAINT:
+      BOOL fChecked = GetCheck() & BST_CHECKED;
+      BOOL fHot = lpNMCustomDraw->uItemState & CDIS_HOT;
+      BOOL fFocus = lpNMCustomDraw->uItemState & CDIS_FOCUS;
+      DrawButton(lpNMCustomDraw->hdr.hwndFrom, lpNMCustomDraw->hdc,
+                 &lpNMCustomDraw->rc, fChecked, fHot, fFocus);
+  }
+}
+
+void CButtonExtn::DrawButton(HWND hWnd, HDC hDC, RECT *pRect, BOOL fChecked, BOOL fHot, BOOL fFocus)
+{
+  // Code originally by Nikita Leontiev in answer to "Change checkBox text color Win32"
+  // in MS's Forum: "Visual Studio Developer Center > Visual Studio vNext Forums > Visual C++ General"
+  // Modified for MFC, Checkbox and Radio buttons by DK
+
+  int nWidth = pRect -> right - pRect -> left;
+  int nHeight = pRect -> bottom - pRect -> top;
+
+  HDC hMemDC = CreateCompatibleDC(hDC);
+  HBITMAP hBitmap = CreateCompatibleBitmap(hDC, nWidth, nHeight);
+  SelectObject(hMemDC, hBitmap);
+
+  RECT rFillRect = {0, 0, nWidth, nHeight};
+
+  HTHEME hTheme = OpenThemeData(hWnd, L"BUTTON");
+  int nStateID(0);
+
+  if (m_type == BS_AUTOCHECKBOX) {
+    nStateID = (fChecked) ? CBS_CHECKEDNORMAL : CBS_UNCHECKEDNORMAL;
+    if (fHot)
+      nStateID = (fChecked) ? CBS_CHECKEDHOT : CBS_UNCHECKEDHOT;
+  } else {
+    nStateID = (fChecked) ? RBS_CHECKEDNORMAL : RBS_UNCHECKEDNORMAL;
+    if (fHot)
+      nStateID = (fChecked) ? RBS_CHECKEDHOT : RBS_UNCHECKEDHOT;
+  }
+
+  //If bg color isn't set, try get backgroung color from current theme
+  if (m_bUseBkgColour) {
+    FillRect(hMemDC, &rFillRect, CreateSolidBrush(GetSysColor(m_icolour)));
+  }
+  else { 
+    // Don't check IsThemeBackgroundPartiallyTransparent because it return false for BP_CHECKBOX
+    DrawThemeParentBackground(hWnd, hMemDC, &rFillRect);
+  }
+
+  RECT rIconRect = {0, 0, 13, nHeight};
+  DrawThemeBackground(hTheme, hMemDC, m_type == BS_AUTOCHECKBOX ? BP_CHECKBOX : BP_RADIOBUTTON,
+                      nStateID, &rIconRect, NULL);
+  CloseThemeData(hTheme);
+
+  RECT rTextRect = {16, 0, nWidth - 16, nHeight};
+  SetBkMode(hMemDC, TRANSPARENT);
+  if (m_bUseTextColour)
+    SetTextColor(hMemDC, m_crfText);
+
+  SelectObject(hMemDC, (HFONT)GetStockObject(DEFAULT_GUI_FONT));
+
+  if (m_caption.IsEmpty()) {
+    GetWindowText(m_caption);
+    SetWindowText(L"");
+  }
+
+  DrawText(hMemDC, m_caption, m_caption.GetLength(), &rTextRect, DT_SINGLELINE | DT_VCENTER);
+
+  if (fFocus){
+    DrawText(hMemDC, m_caption, m_caption.GetLength(), &rTextRect, DT_SINGLELINE | DT_VCENTER | DT_CALCRECT);
+    rTextRect.left--;
+    rTextRect.right++;
+    DrawFocusRect(hMemDC, &rTextRect);
+  }
+
+  BitBlt(hDC, 0, 0, nWidth, nHeight, hMemDC, 0, 0, SRCCOPY);
+
+  DeleteObject(hBitmap);
+  DeleteDC(hMemDC);
+}
+
+/////////////////////////////////////////////////////////////////////////////
+// CButtonExtn
+
+CButtonBitmapExtn::CButtonBitmapExtn()
+{
+}
+
+CButtonBitmapExtn::~CButtonBitmapExtn()
+{
+}
+
+BEGIN_MESSAGE_MAP(CButtonBitmapExtn, CButton)
+  //{{AFX_MSG_MAP(CButtonBitmapExtn)
+  //}}AFX_MSG_MAP
+END_MESSAGE_MAP()
+
+void CButtonBitmapExtn::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
+{
+  CDC dc;
+  dc.Attach(lpDrawItemStruct->hDC);
+
+  CBitmap bmp;
+  bmp.LoadBitmap(m_IDB);
+
+  BITMAP bitMapInfo;
+  bmp.GetBitmap(&bitMapInfo);
+
+  CDC memDC;
+  memDC.CreateCompatibleDC(&dc);
+
+  memDC.SelectObject(&bmp);
+  int bmw = bitMapInfo.bmWidth;
+  int bmh = bitMapInfo.bmHeight;
+
+  // Draw button image transparently
+  ::TransparentBlt(dc.GetSafeHdc(), 0, 0, bmw, bmh, memDC.GetSafeHdc(), 0, 0, bmw, bmh, m_cfMAsk);
+}
