@@ -102,8 +102,12 @@ DECLARE_HANDLE(HDROP);
 // Update current filters whilst SetFilters dialog is open
 #define PWS_MSG_EXECUTE_FILTERS         (WM_APP + 60)
 
+// Notification from tree control that a file was dropped on it
+#define PWS_MSG_DROPPED_FILE            (WM_APP + 65)
+
 /* Message to get Virtual Keyboard buffer.  Here for doc. only. See VKeyBoardDlg.h
 #define PWS_MSG_INSERTBUFFER            (WM_APP + 70)
+#define PWS_MSG_RESETTIMER              (WM_APP + 71)
 */
 
 
@@ -180,11 +184,16 @@ enum PopupMenus {FILEMENU = 0, EXPORTMENU, IMPORTMENU,
                  HELPMENU, NUMPOPUPMENUS};
 
 // Index values for which dialog to show during GetAndCheckPassword
-enum {GCP_FIRST      = 0,   // At startup of PWS
-      GCP_NORMAL     = 1,   // Only OK, CANCEL & HELP buttons
-      GCP_RESTORE    = 2,   // Only OK, CANCEL & HELP buttons
-      GCP_WITHEXIT   = 3,   // OK, CANCEL, EXIT & HELP buttons
-      GCP_CHANGEMODE = 4};  // Only OK, CANCEL & HELP buttons
+enum {
+      // Normal dialogs
+      GCP_FIRST         = 0,   // At startup of PWS
+      GCP_NORMAL        = 1,   // Only OK, CANCEL & HELP buttons
+      GCP_RESTORE       = 2,   // Only OK, CANCEL & HELP buttons
+      GCP_WITHEXIT      = 3,   // OK, CANCEL, EXIT & HELP buttons
+      GCP_CHANGEMODE    = 4,   // Only OK, CANCEL & HELP buttons
+
+      NUM_PER_ENVIRONMENT = 5,
+};
 
 // GCP read only flags - tested via AND, set via OR, must be power of 2.
 enum {GCP_READONLY = 1,
@@ -194,6 +203,7 @@ enum {GCP_READONLY = 1,
 class CDDObList;
 class ExpiredList;
 class CAddEdit_PropertySheet;
+class CPasskeyEntry;
 
 //-----------------------------------------------------------------------------
 class DboxMain : public CDialog, public UIInterFace
@@ -294,6 +304,7 @@ public:
   bool IsDBReadOnly() const {return m_core.IsReadOnly();}
   void SetStartSilent(bool state);
   void SetStartClosed(bool state) {m_IsStartClosed = state;}
+  void SetDBInitiallyRO(bool state) {m_bDBInitiallyRO = state;}
   void MakeRandomPassword(StringX &password, PWPolicy &pwp, bool bIssueMsg = false);
   BOOL LaunchBrowser(const CString &csURL, const StringX &sxAutotype,
                      const std::vector<size_t> &vactionverboffsets,
@@ -580,6 +591,7 @@ public:
   LRESULT OnDragAutoType(WPARAM wParam, LPARAM lParam);
   LRESULT OnExecuteFilters(WPARAM wParam, LPARAM lParam);
   LRESULT OnApplyEditChanges(WPARAM wParam, LPARAM lParam);
+  LRESULT OnDroppedFile(WPARAM wParam, LPARAM lParam);
 
   BOOL PreTranslateMessage(MSG* pMsg);
 
@@ -755,7 +767,6 @@ public:
   afx_msg void OnRunCommand();
   afx_msg void OnColumnPicker();
   afx_msg void OnResetColumns();
-  afx_msg void OnDropFiles(HDROP hDrop);
   afx_msg void OnColumnClick(NMHDR *pNotifyStruct, LRESULT *pLResult);
   afx_msg void OnUpdateNSCommand(CCmdUI *pCmdUI);  // Make entry unsupported (grayed out)
   afx_msg void OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu);
@@ -815,6 +826,8 @@ private:
   StringX m_BrowseURL; // set by OnContextMenu(), used by OnBrowse()
   PWScore &m_core;
 
+  CPasskeyEntry *m_pPasskeyEntryDlg;
+
   bool m_IsStartSilent;
   bool m_IsStartClosed;
   bool m_bStartHiddenAndMinimized;
@@ -954,7 +967,6 @@ private:
 
   // Workstation Locked
   bool m_bWSLocked, m_bWTSRegistered, m_bBlockShutdown;
-  DWORD m_WindowsMajorVersion, m_WindowsMinorVersion;
 
   // Need this in case not running on Vista or later
   HMODULE m_hUser32;
