@@ -312,6 +312,11 @@ SCODE CPWTreeCtrl::GiveFeedback(DROPEFFECT )
 DROPEFFECT CPWTreeCtrl::OnDragEnter(CWnd* , COleDataObject* pDataObject,
                                     DWORD dwKeyState, CPoint )
 {
+  if (pDataObject->IsDataAvailable(CF_HDROP, NULL)) {
+    pws_os::Trace(L"CPWTreeCtrl::OnDragEnter() Found a file!\n");
+    return DROPEFFECT_MOVE;
+  }
+
   // Is it ours?
   if (!pDataObject->IsDataAvailable(m_tcddCPFID, NULL)) 
     return DROPEFFECT_NONE;
@@ -333,6 +338,11 @@ DROPEFFECT CPWTreeCtrl::OnDragEnter(CWnd* , COleDataObject* pDataObject,
 DROPEFFECT CPWTreeCtrl::OnDragOver(CWnd* pWnd , COleDataObject* pDataObject,
                                    DWORD dwKeyState, CPoint point)
 {
+  if (pDataObject->IsDataAvailable(CF_HDROP, NULL)) {
+    pws_os::Trace(L"CPWTreeCtrl::OnDragOver() Found a file!\n");
+    return DROPEFFECT_MOVE;
+  }
+
   // Is it ours?
   if (!pDataObject->IsDataAvailable(m_tcddCPFID, NULL)) 
     return DROPEFFECT_NONE;
@@ -1240,6 +1250,41 @@ BOOL CPWTreeCtrl::OnDrop(CWnd * , COleDataObject *pDataObject,
   // before every return, otherwise next mouse/keybord selection 
   // will be treated as drop target selections
 
+  if (pDataObject->IsDataAvailable(CF_HDROP, NULL)) {
+    HGLOBAL hg;
+    HDROP hdrop;
+    UINT nFiles;
+    TCHAR szDraggedFile[MAX_PATH];
+
+    pws_os::Trace(L"CPWTreeCtrl::OnDrop() Found a file!\n");
+    hg = pDataObject->GetGlobalData(CF_HDROP);
+    if (hg == NULL) {
+      pws_os::Trace(L"CPWTreeCtrl::OnDrop() No global data\n");
+      goto done;
+    }
+    hdrop = HDROP(GlobalLock(hg));
+    if (hdrop == NULL) {
+      pws_os::Trace(L"CPWTreeCtrl::OnDrop() Could not lock global data\n");
+      GlobalUnlock(hg);
+      goto done;
+    }
+    nFiles = DragQueryFile(hdrop, UINT(-1), NULL, 0);
+    // Support exactly one file being dropped
+    if (nFiles != 1) {
+      pws_os::Trace(L"CPWTreeCtrl::OnDrop(): %d files dropped\n", nFiles);
+      GlobalUnlock(hg);
+      goto done;
+    }
+    DragQueryFile(hdrop, 0, szDraggedFile, MAX_PATH);
+    pws_os::Trace(L"CPWTreeCtrl::OnDrop(): %s was dropped\n", szDraggedFile);
+    GlobalUnlock(hg);
+    m_droppedFile = szDraggedFile;
+    app.GetMainDlg()->PostMessage(PWS_MSG_DROPPED_FILE);
+  done:
+    SelectDropTarget(NULL);
+    return FALSE;
+  }
+
   // Is it ours?
   if (!pDataObject->IsDataAvailable(m_tcddCPFID, NULL)) {
     SelectDropTarget(NULL);
@@ -1321,11 +1366,7 @@ BOOL CPWTreeCtrl::OnDrop(CWnd * , COleDataObject *pDataObject,
   int iDDType;
   long lBufLen;
 
-#if (_MSC_VER >= 1400)
   sscanf_s((char *)pData, OLE_HDR_FMT, &lPid, &iDDType, &lBufLen);
-#else
-  sscanf((char *)pData, OLE_HDR_FMT, &lPid, &iDDType, &lBufLen);
-#endif
   pData += OLE_HDR_LEN; // so ProcessData won't sweat
 
   // NULL-ness of pil is also a good indicator of intra/inter-ness
@@ -1433,7 +1474,7 @@ BOOL CPWTreeCtrl::OnDrop(CWnd * , COleDataObject *pDataObject,
           pmulticmds->Add(DBEmptyGroupsCommand::Create(app.GetCore(), sxGroup,
                                                        DBEmptyGroupsCommand::EG_DELETE));
         }
-		
+
         app.GetMainDlg()->Execute(pmulticmds);
       } else
         if (dropEffect == DROPEFFECT_COPY) {
@@ -2086,12 +2127,7 @@ BOOL CPWTreeCtrl::RenderTextData(CLIPFORMAT &cfFormat, HGLOBAL* phGlobal)
     dwBufLen = (ilen + 1) * sizeof(wchar_t);
     lpszW = new WCHAR[ilen + 1];
     //pws_os::Trace(L"lpszW allocated %p, size %d\n", lpszW, dwBufLen);
-#if (_MSC_VER >= 1400)
     (void) wcsncpy_s(lpszW, ilen + 1, cs_dragdata, ilen);
-#else
-    (void)wcsncpy(lpszW, cs_dragdata, ilen);
-    lpszW[ilen] = L'\0';
-#endif
   } else {
     // They want it in ASCII - use lpszW temporarily
     lpszW = cs_dragdata.GetBuffer(ilen + 1);
@@ -2207,12 +2243,8 @@ BOOL CPWTreeCtrl::RenderAllData(HGLOBAL* phGlobal)
 
   char header[OLE_HDR_LEN + 1];
   // Note: GetDDType will return either FROMTREE or FROMTREE_R
-#if (_MSC_VER >= 1400)
   sprintf_s(header, sizeof(header),
             OLE_HDR_FMT, GetCurrentProcessId(), GetDDType(), lBufLen);
-#else
-  sprintf(header, OLE_HDR_FMT, GetCurrentProcessId(), GetDDType(), lBufLen);
-#endif
   CMemFile mf;
   mf.Write(header, OLE_HDR_LEN);
   mf.Write(buffer, lBufLen);
