@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2011 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -16,127 +16,121 @@ down the streetsky.  [Groucho Marx]
 #include "PasswordSafe.h"
 #include "PWFileDialog.h"
 #include "ThisMfcApp.h"
+#include "PasskeyEntry.h"
+#include "PasskeySetup.h"
+#include "Fonts.h"
+#include "DboxMain.h" // for CheckPasskey()
+#include "PWSversion.h"
 #include "GeneralMsgBox.h"
+#include "SDThread.h"
 
 #include "core/PwsPlatform.h"
 #include "core/Pwsdirs.h"
 #include "core/pwsprefs.h"
+#include "core/PWScore.h"
+#include "core/PWSfile.h"
+#include "core/Util.h"
+#include "core/core.h"
 
 #include "os/file.h"
-#include "os/env.h"
 #include "os/dir.h"
 
 #include "VirtualKeyboard/VKeyBoardDlg.h"
 
-#if defined(POCKET_PC)
-#include "pocketpc/resource.h"
-#include "pocketpc/PocketPC.h"
-#else
 #include "resource.h"
 #include "resource3.h"  // String resources
-#endif
 
 #include "SecString.h"
 
 #include "SysColStatic.h"
 
-#include "PasskeyEntry.h"
-#include "PwFont.h"
-#include "TryAgainDlg.h"
-#include "DboxMain.h" // for CheckPasskey()
-#include "PasskeySetup.h"
-
-#include "core/Util.h"
-
 #include <iomanip>  // For setbase and setw
 
-static wchar_t PSSWDCHAR = L'*';
-
 // See DboxMain.h for the relevant enum
-int CPasskeyEntry::dialog_lookup[4] = {
+int CPasskeyEntry::dialog_lookup[10] = {
+  // Normal dialogs
   IDD_PASSKEYENTRY_FIRST,          // GCP_FIRST
   IDD_PASSKEYENTRY,                // GCP_NORMAL
   IDD_PASSKEYENTRY,                // GCP_RESTORE
-  IDD_PASSKEYENTRY_WITHEXIT};      // GCP_WITHEXIT
+  IDD_PASSKEYENTRY_WITHEXIT,       // GCP_WITHEXIT
+  IDD_PASSKEYENTRY,                // GCP_CHANGEMODE
+  // Secure Desktop dialogs
+  IDD_PASSKEYENTRY_FIRST_SD,       // GCP_FIRST_SD
+  IDD_PASSKEYENTRY_SD,             // GCP_NORMAL_SD
+  IDD_PASSKEYENTRY_SD,             // GCP_RESTORE_SD
+  IDD_PASSKEYENTRY_WITHEXIT_SD,    // GCP_WITHEXIT_SD
+  IDD_PASSKEYENTRY_SD              // GCP_CHANGEMODE_SD
+};
 
 //-----------------------------------------------------------------------------
 CPasskeyEntry::CPasskeyEntry(CWnd* pParent, const CString& a_filespec, int index,
-                             bool bReadOnly, bool bForceReadOnly, bool bHideReadOnly)
-  : CPWDialog(dialog_lookup[index], pParent),
-  m_index(index),
+                             bool bReadOnly, bool bForceReadOnly, bool bHideReadOnly,
+                             bool bUseSecureDesktop)
+  : CPKBaseDlg(dialog_lookup[index + (bUseSecureDesktop ? NUM_PER_ENVIRONMENT : 0)], pParent, bUseSecureDesktop),
   m_filespec(a_filespec), m_orig_filespec(a_filespec),
   m_tries(0),
   m_status(TAR_INVALID),
   m_PKE_ReadOnly(bReadOnly ? TRUE : FALSE),
-  m_bForceReadOnly(bForceReadOnly),
-  m_bHideReadOnly(bHideReadOnly),
-  m_pVKeyBoardDlg(NULL)
+  m_bForceReadOnly(bForceReadOnly), m_bHideReadOnly(bHideReadOnly),
+  m_yubi_sk(NULL)
 {
+  m_index = index;
+
   DBGMSG("CPasskeyEntry()\n");
   if (m_index == GCP_FIRST) {
     DBGMSG("** FIRST **\n");
   }
 
-  m_passkey = L"";
   m_hIcon = app.LoadIcon(IDI_CORNERICON);
-  m_message = a_filespec;
+  m_SelectedDatabase = a_filespec;
 
-  m_pDbx = dynamic_cast<DboxMain *>(pParent);
-  ASSERT(m_pDbx != NULL);
+  PWSversion *pPWSver = PWSversion::GetInstance();
+  int nMajor = pPWSver->GetMajor();
+  int nMinor = pPWSver->GetMinor();
+  int nBuild = pPWSver->GetBuild();
+  CString csSpecialBuild = pPWSver->GetSpecialBuild();
 
-  m_pctlPasskey = new CSecEditExtn;
-  if (pws_os::getenv("PWS_PW_MODE", false) == L"NORMAL")
-    m_pctlPasskey->SetSecure(false);
+  if (nBuild == 0)
+    m_appversion.Format(L"V%d.%02d%s", nMajor, nMinor, csSpecialBuild);
+  else
+    m_appversion.Format(L"V%d.%02d.%02d%s", nMajor, nMinor, nBuild, csSpecialBuild);
 }
 
 CPasskeyEntry::~CPasskeyEntry()
 {
   ::DestroyIcon(m_hIcon);
-  delete m_pctlPasskey;
-  
-  if (m_pVKeyBoardDlg != NULL) {
-    // Save Last Used Keyboard
-    UINT uiKLID = m_pVKeyBoardDlg->GetKLID();
-    std::wostringstream os;
-    os.fill(L'0');
-    os << std::nouppercase << std::hex << std::setw(8) << uiKLID;
-    StringX cs_KLID = os.str().c_str();
-    PWSprefs::GetInstance()->SetPref(PWSprefs::LastUsedKeyboard, cs_KLID);
 
-    m_pVKeyBoardDlg->DestroyWindow();
-    delete m_pVKeyBoardDlg;
+  if (m_yubi_sk != NULL) {
+    trashMemory(m_yubi_sk, PWSfile::HeaderRecord::YUBI_SK_LEN);
+    delete[] m_yubi_sk;
   }
 }
 
 void CPasskeyEntry::DoDataExchange(CDataExchange* pDX)
 {
-  CPWDialog::DoDataExchange(pDX);
-
-  // Can't use DDX_Text for CSecEditExtn
-  m_pctlPasskey->DoDDX(pDX, m_passkey);
-
-#if !defined(POCKET_PC)
-  if (m_index == GCP_FIRST)
-    DDX_Control(pDX, IDC_STATIC_LOGOTEXT, m_ctlLogoText);
-#endif
+  CPKBaseDlg::DoDataExchange(pDX);
 
   //{{AFX_DATA_MAP(CPasskeyEntry)
-#if !defined(POCKET_PC)
+  if (m_index == GCP_FIRST) {
+    DDX_Control(pDX, IDC_STATIC_LOGOTEXT, m_ctlLogoText);
+    DDX_Control(pDX, IDC_DATABASECOMBO, m_MRU_combo);
+  }
+
   DDX_Control(pDX, IDC_STATIC_LOGO, m_ctlLogo);
-  DDX_Control(pDX, IDOK, m_ctlOK);
-#endif
-  DDX_Control(pDX, IDC_PASSKEY, *m_pctlPasskey);
-  DDX_Text(pDX, IDC_MESSAGE, m_message);
+  DDX_Text(pDX, IDC_SELECTED_DATABASE, m_SelectedDatabase);
   DDX_Check(pDX, IDC_READONLY, m_PKE_ReadOnly);
 
-  if (m_index == GCP_FIRST)
-    DDX_Control(pDX, IDC_DATABASECOMBO, m_MRU_combo);
-
+  if (!m_bUseSecureDesktop)
+    DDX_Control(pDX, IDOK, m_ctlOK);
+  else
+    DDX_Control(pDX, IDC_ENTERCOMBINATION, m_ctlEnterCombination);
   //}}AFX_DATA_MAP
 }
 
-BEGIN_MESSAGE_MAP(CPasskeyEntry, CPWDialog)
+BEGIN_MESSAGE_MAP(CPasskeyEntry, CPKBaseDlg)
   //{{AFX_MSG_MAP(CPasskeyEntry)
+  ON_WM_DESTROY()
+  ON_WM_TIMER()
   ON_BN_CLICKED(ID_HELP, OnHelp)
   ON_BN_CLICKED(IDC_CREATE_DB, OnCreateDb)
   ON_BN_CLICKED(IDC_EXIT, OnExit)
@@ -145,10 +139,8 @@ BEGIN_MESSAGE_MAP(CPasskeyEntry, CPWDialog)
   ON_BN_CLICKED(IDC_BTN_BROWSE, OnOpenFileBrowser)
   ON_MESSAGE(PWS_MSG_INSERTBUFFER, OnInsertBuffer)
   ON_STN_CLICKED(IDC_VKB, OnVirtualKeyboard)
-#if defined(POCKET_PC)
-  ON_EN_SETFOCUS(IDC_PASSKEY, OnPasskeySetfocus)
-  ON_EN_KILLFOCUS(IDC_PASSKEY, OnPasskeyKillfocus)
-#endif
+  ON_BN_CLICKED(IDC_YUBIKEY_BTN, OnYubikeyBtn)
+  ON_BN_CLICKED(IDC_ENTERCOMBINATION, OnEnterCombination)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -167,75 +159,74 @@ static CString NarrowPathText(const CString &text)
 
 BOOL CPasskeyEntry::OnInitDialog(void)
 {
-#if defined(POCKET_PC)
-  // If displaying IDD_PASSKEYENTRY_FIRST then bypass superclass and go
-  // directly to CDialog::OnInitDialog() and display the dialog fullscreen
-  // otherwise display as a centred dialogue.
-  if (m_nIDHelp == IDD) {
-    CDialog::OnInitDialog();
-  } else {
-    CPWDialog::OnInitDialog();
+  CPKBaseDlg::OnInitDialog();
+
+  if (!m_bUseSecureDesktop) {
+    Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
+
+    m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
   }
-#else
-  CPWDialog::OnInitDialog();
-#endif
 
-  ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
-
-  m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
-
-  switch(m_index) {
-    case GCP_FIRST:
-      // At start up - give the user the option unless file is R/O
-      GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
-      GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
-      GetDlgItem(IDC_VERSION)->SetWindowText(m_appversion);
+  switch (m_index) {
+  case GCP_FIRST:
+    // At start up - give the user the option unless file is R-O
+    GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
+    GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_VERSION)->SetWindowText(m_appversion);
 #ifdef DEMO
-      GetDlgItem(IDC_SPCL_TXT)->
-         SetWindowText(CString(MAKEINTRESOURCE(IDS_DEMO)));
+    GetDlgItem(IDC_SPCL_TXT)->
+      SetWindowText(CString(MAKEINTRESOURCE(IDS_DEMO)));
 #endif
-      break;
-    case GCP_NORMAL:
-      // otherwise during open - user can - again unless file is R/O
-      if (m_bHideReadOnly) {
-        GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
-        GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
-      } else {
-        GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
-        GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
-      }
-      break;
-    case GCP_RESTORE:
-    case GCP_WITHEXIT:
-      // on Restore - user can't change status
+    break;
+  case GCP_NORMAL:
+    // otherwise during open - user can - again unless file is R-O
+    if (m_bHideReadOnly) {
       GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
       GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
-      break;
-    default:
-      ASSERT(FALSE);
+    }
+    else {
+      GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
+      GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
+    }
+    break;
+  case GCP_RESTORE:
+  case GCP_WITHEXIT:
+    GetDlgItem(IDC_READONLY)->EnableWindow(m_bForceReadOnly ? FALSE : TRUE);
+    GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
+    break;
+  case GCP_CHANGEMODE:
+    GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
+    GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
+    break;
+  default:
+    ASSERT(FALSE);
   }
 
   // Only show virtual Keyboard menu if we can load DLL
-  if (!CVKeyBoardDlg::IsOSKAvailable()) {
+  if (!m_bUseSecureDesktop && !m_bVKAvailable) {
     GetDlgItem(IDC_VKB)->ShowWindow(SW_HIDE);
     GetDlgItem(IDC_VKB)->EnableWindow(FALSE);
   }
 
-  if (m_message.IsEmpty() && m_index == GCP_FIRST) {
-    m_pctlPasskey->EnableWindow(FALSE);
-#if !defined(POCKET_PC)
-    m_ctlOK.EnableWindow(FALSE);
-#endif
-    m_message.LoadString(IDS_NOCURRENTSAFE);
+  if (m_SelectedDatabase.IsEmpty()) {
+    if (m_index == GCP_FIRST) {
+      if (m_bUseSecureDesktop) {
+        m_pctlPasskey->EnableWindow(FALSE);
+        m_ctlOK.EnableWindow(TRUE);
+        m_SelectedDatabase.LoadString(IDS_NOCURRENTSAFE);
+      }
+      else
+        m_ctlEnterCombination.EnableWindow(FALSE);
+    }
   }
 
   if (m_index == GCP_FIRST) {
-    GetDlgItem(IDC_MESSAGE)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_SELECTED_DATABASE)->ShowWindow(SW_HIDE);
 
     CRecentFileList *mru = app.GetMRU();
     ASSERT(mru != NULL);
 
-    const int N = mru->GetSize();
+    int N = mru->GetSize();
 
     std::vector<CSecString> cs_tooltips;
 
@@ -255,25 +246,34 @@ BOOL CPasskeyEntry::OnInitDialog(void)
           m_MRU_combo.SetItemData(li, i);
       }
     }
+    if ((N > 0) || !m_filespec.IsEmpty()) {
+      // Add an empty row to allow NODB
+      int li = m_MRU_combo.AddString(L"");
+      if (li != CB_ERR && li != CB_ERRSPACE) {
+        m_MRU_combo.SetItemData(li, DWORD_PTR(-2)); // -1 already taken, but a < 0 value is easier to check than N
+        CString cs_empty(MAKEINTRESOURCE(IDS_EMPTY_DB));
+        cs_tooltips.push_back(cs_empty);
+        N++; // for SetHeight
+      }
+    }
+
     if (N > 0) {
       SetHeight(N);
     }
+
     m_MRU_combo.SetToolTipStrings(cs_tooltips);
   }
 
   /*
-   * this bit makes the background come out right on
-   * the bitmaps
+   * this bit makes the background come out right on the bitmaps
    */
 
-#if !defined(POCKET_PC)
   if (m_index == GCP_FIRST) {
     m_ctlLogoText.ReloadBitmap(IDB_PSLOGO);
-    m_ctlLogo.ReloadBitmap(IDB_CLOGO);
+    m_ctlLogo.ReloadBitmap(m_bUseSecureDesktop ? IDB_CLOGO_SMALL : IDB_CLOGO);
   } else {
     m_ctlLogo.ReloadBitmap(IDB_CLOGO_SMALL);
   }
-#endif
 
   // Set the icon for this dialog.  The framework does this automatically
   //  when the application's main window is not a dialog
@@ -290,7 +290,8 @@ BOOL CPasskeyEntry::OnInitDialog(void)
     SetWindowPos(&CWnd::wndTopMost, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
     SetActiveWindow();
     SetForegroundWindow();
-    m_pctlPasskey->SetFocus();
+    if (!m_bUseSecureDesktop)
+      m_pctlPasskey->SetFocus();
     return FALSE;
   }
 
@@ -299,34 +300,16 @@ BOOL CPasskeyEntry::OnInitDialog(void)
 
   // If the dbase field's !empty, the user most likely will want to enter
   // a password:
-  if (m_index == 0 && !m_filespec.IsEmpty()) {
-    m_pctlPasskey->SetFocus();
+  if (m_index == GCP_FIRST && !m_filespec.IsEmpty()) {
+    m_MRU_combo.SetEditSel(-1, -1);
+    if (!m_bUseSecureDesktop)
+      m_pctlPasskey->SetFocus();
     return FALSE;
   }
 
   return TRUE;
 }
 
-#if defined(POCKET_PC)
-/************************************************************************/
-/* Restore the state of word completion when the password field loses   */
-/* focus.                                                               */
-/************************************************************************/
-void CPasskeyEntry::OnPasskeyKillfocus()
-{
-  EnableWordCompletion(m_hWnd);
-}
-
-/************************************************************************/
-/* When the password field is activated, pull up the SIP and disable    */
-/* word completion.                                                     */
-/************************************************************************/
-void CPasskeyEntry::OnPasskeySetfocus()
-{
-  DisableWordCompletion(m_hWnd);
-}
-#endif
-
 void CPasskeyEntry::OnCreateDb()
 {
   // 1. Get a filename from a file dialog box
@@ -381,16 +364,28 @@ void CPasskeyEntry::OnCreateDb()
   }
 
   // 2. Get a password
-  CPasskeySetup dbox_pksetup(this);
-  rc = dbox_pksetup.DoModal();
+  bool bUseSecureDesktop = PWSprefs::GetInstance()->GetPref(PWSprefs::UseSecureDesktop);
+
+  do
+  {
+    CPasskeySetup pksetup(this, *app.GetCore(), bUseSecureDesktop);
+    rc = pksetup.DoModal();
+
+    if (rc == IDOK)
+      m_passkey = pksetup.GetPassKey();
+
+    // In case user wanted to toggle Secure Desktop
+    bUseSecureDesktop = !bUseSecureDesktop;
+  } while (rc == INT_MAX);
 
   if (rc != IDOK)
     return;  //User cancelled password entry
 
   // 3. Set m_filespec && m_passkey to returned value!
   m_filespec = newfile;
-  m_passkey = dbox_pksetup.m_passkey;
-  ((CEdit*)GetDlgItem(IDC_PASSKEY))->SetWindowText(m_passkey);
+  if (!m_bUseSecureDesktop)
+    ((CEdit*)GetDlgItem(IDC_PASSKEY))->SetWindowText(m_passkey);
+
   m_status = TAR_NEW;
   CPWDialog::OnOK();
 }
@@ -411,6 +406,12 @@ void CPasskeyEntry::OnOK()
 {
   UpdateData(TRUE);
 
+  if (m_filespec.IsEmpty()) {
+    m_status = TAR_OPEN_NODB;
+    CPWDialog::OnCancel();
+    return;
+  }
+
   CGeneralMsgBox gmb;
   if (m_passkey.IsEmpty()) {
     gmb.AfxMessageBox(IDS_CANNOTBEBLANK);
@@ -431,43 +432,58 @@ void CPasskeyEntry::OnOK()
 void CPasskeyEntry::ProcessPhrase()
 {
   CGeneralMsgBox gmb;
-  if (m_pDbx->CheckPasskey(LPCWSTR(m_filespec), LPCWSTR(m_passkey)) != PWScore::SUCCESS) {
-    if (m_tries >= 2) {
-      CTryAgainDlg errorDlg(this);
 
-      INT_PTR nResponse = errorDlg.DoModal();
-      if (nResponse == IDOK) {
-      } else if (nResponse == IDCANCEL) {
-        m_status = errorDlg.GetCancelReturnValue();
-        if (m_status == TAR_OPEN) { // open another
-          PostMessage(WM_COMMAND, IDC_BTN_BROWSE);
-          return;
-        } else if (m_status == TAR_NEW) { // create new
-          PostMessage(WM_COMMAND, IDC_CREATE_DB);
-          return;
-        }
-        CPWDialog::OnCancel();
-      }
-    } else {
-      m_tries++;
-      gmb.AfxMessageBox(IDS_INCORRECTKEY);
+  switch (GetMainDlg()->CheckPasskey(LPCWSTR(m_filespec), LPCWSTR(m_passkey))) {
+  case PWScore::SUCCESS: {
+    // OnOK clears the passkey, so we save it
+    const CSecString save_passkey = m_passkey;
+    // Try to change read-only state if user changed checkbox:
+    // r/w -> r-o always succeeds
+    // r-o -> r/w may fail
+    // Note that if file is read-only, m_bForceReadOnly is true -> checkbox
+    // is disabled -> don't need to worry about that.
+    if ((m_index == GCP_RESTORE || m_index == GCP_WITHEXIT) && 
+        (m_PKE_ReadOnly == TRUE) == pws_os::IsLockedFile(LPCWSTR(m_filespec))) {
+      GetMainDlg()->ChangeMode(false); // false means
+      //                           "don't prompt use for password", as we just got it.
+    }
+    CPWDialog::OnOK();
+    m_passkey = save_passkey;
+  }
+    break;
+  case PWScore::WRONG_PASSWORD:
+    if (m_tries++ >= 2) { // too many tries
+      CString cs_toomany;
+      cs_toomany.Format(IDS_TOOMANYTRIES, m_tries);
+      gmb.AfxMessageBox(cs_toomany);
+    }
+    else { // try again
+      gmb.AfxMessageBox(m_index == GCP_CHANGEMODE ? IDS_BADPASSKEY : IDS_INCORRECTKEY);
+    }
+    if (!m_bUseSecureDesktop) {
       m_pctlPasskey->SetSel(MAKEWORD(-1, 0));
       m_pctlPasskey->SetFocus();
     }
-  } else {
-    CPWDialog::OnOK();
+    break;
+  case PWScore::READ_FAIL:
+    gmb.AfxMessageBox(IDSC_FILE_UNREADABLE);
+    CPWDialog::OnCancel();
+    break;
+  case PWScore::TRUNCATED_FILE:
+    gmb.AfxMessageBox(IDSC_FILE_TRUNCATED);
+    CPWDialog::OnCancel();
+    break;
+  default:
+    ASSERT(0);
+    gmb.AfxMessageBox(IDSC_UNKNOWN_ERROR);
+    CPWDialog::OnCancel();
+    break;
   }
 }
 
 void CPasskeyEntry::OnHelp()
 {
-#if defined(POCKET_PC)
-  CreateProcess(L"PegHelp.exe", L"pws_ce_help.html#comboentry", NULL, NULL, FALSE, 0, NULL, NULL, NULL, NULL);
-#else
-  CString cs_HelpTopic;
-  cs_HelpTopic = app.GetHelpFileName() + L"::/html/create_new_db.html";
-  HtmlHelp(DWORD_PTR((LPCWSTR)cs_HelpTopic), HH_DISPLAY_TOPIC);
-#endif
+  ShowHelp(L"::/html/create_new_db.html");
 }
 
 //-----------------------------------------------------------------------------
@@ -486,6 +502,8 @@ void CPasskeyEntry::UpdateRO()
         m_PKE_ReadOnly = FALSE;
       GetDlgItem(IDC_READONLY)->EnableWindow(TRUE);
     }
+    ((CButton *)GetDlgItem(IDC_READONLY))->SetCheck(m_PKE_ReadOnly == TRUE ? 
+                                                    BST_CHECKED : BST_UNCHECKED);
     UpdateData(FALSE);
   } // !m_bForceReadOnly
 }
@@ -493,8 +511,14 @@ void CPasskeyEntry::UpdateRO()
 void CPasskeyEntry::OnComboEditChange()
 {
   m_MRU_combo.m_edit.GetWindowText(m_filespec);
-  m_pctlPasskey->EnableWindow(TRUE);
-  m_ctlOK.EnableWindow(TRUE);
+  if (!m_bUseSecureDesktop) {
+    m_pctlPasskey->EnableWindow(TRUE);
+    m_ctlOK.EnableWindow(TRUE);
+  }
+  else
+  {
+    m_ctlEnterCombination.EnableWindow(TRUE);
+  }
 
   UpdateRO();
 }
@@ -505,19 +529,33 @@ void CPasskeyEntry::OnComboSelChange()
   int curSel = m_MRU_combo.GetCurSel();
   const int N = mru->GetSize();
 
-  if (curSel == CB_ERR || curSel >= N) {
+  if (curSel == CB_ERR) {
     ASSERT(0);
+  } else if (curSel >= N) {
+    m_filespec = L"";
   } else {
     int i = int(m_MRU_combo.GetItemData(curSel));
     if (i >= 0) // -1 means original m_filespec
       m_filespec = (*mru)[i];
+    else if (i == -2)
+      m_filespec = L"";
     else
       m_filespec = m_orig_filespec;
   }
 
-  m_pctlPasskey->EnableWindow(TRUE);
-  m_pctlPasskey->SetFocus();
-  m_ctlOK.EnableWindow(TRUE);
+  if (!m_bUseSecureDesktop) {
+    m_pctlPasskey->EnableWindow(TRUE);
+    m_pctlPasskey->SetFocus();
+    m_ctlOK.EnableWindow(TRUE);
+  }
+  else
+  {
+    CString csText;
+    csText.LoadString(m_filespec.IsEmpty() ? IDS_OK : IDS_ENTERCOMBINATION);
+
+    m_ctlEnterCombination.SetWindowText(csText);
+    m_ctlEnterCombination.EnableWindow(TRUE);
+  }
 
   UpdateRO();
 }
@@ -542,12 +580,11 @@ void CPasskeyEntry::OnOpenFileBrowser()
       fd.m_ofn.Flags &= ~OFN_READONLY;
 
   std::wstring dir;
-  DboxMain *pDbx = (DboxMain*)GetParent();
-  if (pDbx->GetCurFile().empty())
+  if (GetMainDlg()->GetCurFile().empty())
     dir = PWSdirs::GetSafeDir();
   else {
     std::wstring cdrive, cdir, dontCare;
-    pws_os::splitpath(pDbx->GetCurFile().c_str(), cdrive, cdir, dontCare, dontCare);
+    pws_os::splitpath(GetMainDlg()->GetCurFile().c_str(), cdrive, cdir, dontCare, dontCare);
     dir = cdrive + cdir;
   }
 
@@ -565,14 +602,19 @@ void CPasskeyEntry::OnOpenFileBrowser()
   }
   if (rc == IDOK) {
     m_PKE_ReadOnly = fd.GetReadOnlyPref();
-    ((CButton *)GetDlgItem(IDC_READONLY))->SetCheck(m_PKE_ReadOnly == TRUE ? 
-                                           BST_CHECKED : BST_UNCHECKED);
     m_filespec = fd.GetPathName();
     m_MRU_combo.m_edit.SetWindowText(m_filespec);
-    m_pctlPasskey->EnableWindow(TRUE);
-    m_ctlOK.EnableWindow(TRUE);
-    if (m_pctlPasskey->IsWindowEnabled() == TRUE) {
-      m_pctlPasskey->SetFocus();
+    if (!m_bUseSecureDesktop) {
+      // Normal
+      m_pctlPasskey->EnableWindow(TRUE);
+      if (m_pctlPasskey->IsWindowEnabled() == TRUE) {
+        m_pctlPasskey->SetFocus();
+      }
+      m_ctlOK.EnableWindow(TRUE);
+    }
+    else {
+      // Secure Desktop
+      m_ctlEnterCombination.EnableWindow(TRUE);
     }
     UpdateRO();
   } // rc == IDOK
@@ -606,33 +648,43 @@ void CPasskeyEntry::SetHeight(const int num)
 
 void CPasskeyEntry::OnVirtualKeyboard()
 {
+  // This is used if Secure Desktop isn't!
+  DWORD dwError; //  Define it here to stop warning that local variable is initialized but not referenced later on
+
   // Shouldn't be here if couldn't load DLL. Static control disabled/hidden
-  if (!CVKeyBoardDlg::IsOSKAvailable())
+  if (!m_bVKAvailable)
     return;
 
-  if (m_pVKeyBoardDlg != NULL && m_pVKeyBoardDlg->IsWindowVisible()) {
+  if (m_hwndVKeyBoard != NULL && ::IsWindowVisible(m_hwndVKeyBoard)) {
     // Already there - move to top
-    m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    ::SetWindowPos(m_hwndVKeyBoard, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
     return;
   }
 
   // If not already created - do it, otherwise just reset it
   if (m_pVKeyBoardDlg == NULL) {
     StringX cs_LUKBD = PWSprefs::GetInstance()->GetPref(PWSprefs::LastUsedKeyboard);
-    m_pVKeyBoardDlg = new CVKeyBoardDlg(this, cs_LUKBD.c_str());
-    m_pVKeyBoardDlg->Create(CVKeyBoardDlg::IDD);
+    m_pVKeyBoardDlg = new CVKeyBoardDlg(this->GetSafeHwnd(), this->GetSafeHwnd(), cs_LUKBD.c_str());
+    m_hwndVKeyBoard = CreateDialogParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_SDVKEYBOARD), this->GetSafeHwnd(),
+      (DLGPROC)(m_pVKeyBoardDlg->VKDialogProc), (LPARAM)(m_pVKeyBoardDlg));
+
+    if (m_hwndVKeyBoard == NULL) {
+      dwError = pws_os::IssueError(_T("CreateDialogParam - IDD_SDVKEYBOARD"), false);
+      ASSERT(m_hwndVKeyBoard);
+    }
   } else {
     m_pVKeyBoardDlg->ResetKeyboard();
   }
 
-  // Now show it and make it top
-  m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
-
-  return;
+  // Now show it and make it top & enable it
+  ::SetWindowPos(m_hwndVKeyBoard, HWND_TOP, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
+  ::EnableWindow(m_hwndVKeyBoard, TRUE);
 }
 
 LRESULT CPasskeyEntry::OnInsertBuffer(WPARAM, LPARAM)
 {
+  // This is used if Secure Desktop isn't!
+
   // Update the variables
   UpdateData(TRUE);
 
@@ -659,3 +711,40 @@ LRESULT CPasskeyEntry::OnInsertBuffer(WPARAM, LPARAM)
 
   return 0L;
 }
+
+void CPasskeyEntry::OnYubikeyBtn()
+{
+  UpdateData(TRUE);
+  if (!pws_os::FileExists(m_filespec.GetString())) {
+    CGeneralMsgBox gmb;
+    gmb.AfxMessageBox(IDS_FILEPATHNOTFOUND);
+    if (m_MRU_combo.IsWindowVisible())
+      m_MRU_combo.SetFocus();
+    return;
+  }
+  yubiRequestHMACSha1(m_passkey);
+}
+
+void CPasskeyEntry::OnEnterCombination()
+{
+  if (m_filespec.IsEmpty()) {
+    m_status = TAR_OPEN_NODB;
+    CPWDialog::OnCancel();
+    return;
+  }
+
+  // Get passphrase from Secure Desktop
+  StartThread(IDD_SDGETPHRASE);
+
+  ShowWindow(SW_SHOW);
+
+  if (m_GMP.bPhraseEntered) {
+    ShowWindow(SW_SHOW);
+    m_passkey = m_GMP.sPhrase.c_str();
+    ProcessPhrase();
+  }
+
+  // Just do nothing if no passphrase entered i.e. user pressed cancel
+  // Try to get this seen
+  SetForegroundWindow();
+}
