@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -10,22 +10,17 @@
 
 #include "stdafx.h"
 #include "passwordsafe.h"
+#include "DboxMain.h"
 #include "PasswordPolicyDlg.h"
 #include "GeneralMsgBox.h"
-#include "DboxMain.h"
-#include "ThisMfcApp.h"    // For Help
 
 #include "core/core.h"
 #include "core/PWCharPool.h"
 #include "core/PwsPlatform.h"
 #include "core/PWSprefs.h"
 
-#if defined(POCKET_PC)
-#include "pocketpc/resource.h"
-#else
 #include "resource.h"
 #include "resource3.h"  // String resources
-#endif
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -63,7 +58,7 @@ IMPLEMENT_DYNAMIC(CPasswordPolicyDlg, CPWDialog)
 CPasswordPolicyDlg::CPasswordPolicyDlg(UINT uicaller, CWnd *pParent, bool bLongPPs,
                                        bool bReadOnly, PWPolicy &st_default_pp)
   : CPWDialog(bLongPPs ? CPasswordPolicyDlg::IDD : CPasswordPolicyDlg::IDD_SHORT, pParent),
-  m_uicaller(uicaller), m_bReadOnly(bReadOnly), m_pDbx(NULL), m_password(L""),
+  m_uicaller(uicaller), m_bReadOnly(bReadOnly), m_password(L""),
   m_UseNamedPolicy(FALSE), m_st_default_pp(st_default_pp), m_bLongPPs(bLongPPs)
 {
   m_PWUseLowercase = m_oldPWUseLowercase =
@@ -87,8 +82,19 @@ CPasswordPolicyDlg::CPasswordPolicyDlg(UINT uicaller, CWnd *pParent, bool bLongP
   m_PWSymbolMinLength = m_oldPWSymbolMinLength = m_st_default_pp.symbolminlength;
   m_PWUpperMinLength = m_oldPWUpperMinLength = m_st_default_pp.upperminlength;
 
-  m_Symbols = m_oldSymbols = m_st_default_pp.symbols.c_str();
-  m_UseOwnSymbols = m_oldUseOwnSymbols = m_Symbols.IsEmpty() ? DEFAULT_SYMBOLS : OWN_SYMBOLS;
+  if (!m_st_default_pp.symbols.empty())
+    m_Symbols = m_oldSymbols = m_st_default_pp.symbols.c_str();
+  else {
+    // empty, set to appropriate default
+    stringT st_symbols;
+    if (m_PWEasyVision == TRUE)
+      st_symbols = CPasswordCharPool::GetEasyVisionSymbols();
+    else if (m_PWMakePronounceable == TRUE)
+      st_symbols = CPasswordCharPool::GetPronounceableSymbols();
+    else
+      st_symbols = CPasswordCharPool::GetDefaultSymbols();
+    m_Symbols = m_oldSymbols = st_symbols.c_str();
+  }
 
   // These must be in the same order as their respective Control IDs
   // in CPasswordPolicyDlg::nonHex
@@ -98,89 +104,92 @@ CPasswordPolicyDlg::CPasswordPolicyDlg(UINT uicaller, CWnd *pParent, bool bLongP
   m_pnonHex[3] = &m_PWUseSymbols;
   m_pnonHex[4] = &m_PWEasyVision;
   m_pnonHex[5] = &m_PWMakePronounceable;
-
-  CPasswordCharPool::GetDefaultSymbols(m_std_symbols);
-  CPasswordCharPool::GetEasyVisionSymbols(m_easyvision_symbols);
-  CPasswordCharPool::GetPronounceableSymbols(m_pronounceable_symbols);
 }
 
 CPasswordPolicyDlg::~CPasswordPolicyDlg()
 {
+  m_CopyPswdBitmap.Detach();
 }
 
 void CPasswordPolicyDlg::DoDataExchange(CDataExchange* pDX)
 {
-  CPWDialog::DoDataExchange(pDX);
+    CPWDialog::DoDataExchange(pDX);
 
-  //{{AFX_DATA_MAP(CPasswordPolicyDlg)
-  DDX_Text(pDX, IDC_DEFPWLENGTH, m_PWDefaultLength);
-  DDX_Text(pDX, IDC_MINDIGITLENGTH, m_PWDigitMinLength);
-  DDX_Text(pDX, IDC_MINLOWERLENGTH, m_PWLowerMinLength);
-  DDX_Text(pDX, IDC_MINSYMBOLLENGTH, m_PWSymbolMinLength);
-  DDX_Text(pDX, IDC_MINUPPERLENGTH, m_PWUpperMinLength);
-  DDX_Check(pDX, IDC_USELOWERCASE, m_PWUseLowercase);
-  DDX_Check(pDX, IDC_USEUPPERCASE, m_PWUseUppercase);
-  DDX_Check(pDX, IDC_USEDIGITS, m_PWUseDigits);
-  DDX_Check(pDX, IDC_USESYMBOLS, m_PWUseSymbols);
-  DDX_Check(pDX, IDC_EASYVISION, m_PWEasyVision);
-  DDX_Check(pDX, IDC_USEHEXDIGITS, m_PWUseHexdigits);
-  DDX_Check(pDX, IDC_PRONOUNCEABLE, m_PWMakePronounceable);
-  DDX_Check(pDX, IDC_USENAMED_POLICY, m_UseNamedPolicy);
+    //{{AFX_DATA_MAP(CPasswordPolicyDlg)
+    DDX_Text(pDX, IDC_DEFPWLENGTH, m_PWDefaultLength);
+    DDX_Text(pDX, IDC_MINDIGITLENGTH, m_PWDigitMinLength);
+    DDX_Text(pDX, IDC_MINLOWERLENGTH, m_PWLowerMinLength);
+    DDX_Text(pDX, IDC_MINSYMBOLLENGTH, m_PWSymbolMinLength);
+    DDX_Text(pDX, IDC_MINUPPERLENGTH, m_PWUpperMinLength);
+    DDX_Check(pDX, IDC_USELOWERCASE, m_PWUseLowercase);
+    DDX_Check(pDX, IDC_USEUPPERCASE, m_PWUseUppercase);
+    DDX_Check(pDX, IDC_USEDIGITS, m_PWUseDigits);
+    DDX_Check(pDX, IDC_USESYMBOLS, m_PWUseSymbols);
+    DDX_Check(pDX, IDC_EASYVISION, m_PWEasyVision);
+    DDX_Check(pDX, IDC_USEHEXDIGITS, m_PWUseHexdigits);
+    DDX_Check(pDX, IDC_PRONOUNCEABLE, m_PWMakePronounceable);
+    DDX_Check(pDX, IDC_USENAMED_POLICY, m_UseNamedPolicy);
 
-  DDX_Radio(pDX, IDC_USEDEFAULTSYMBOLS, m_UseOwnSymbols);
-  DDX_Control(pDX, IDC_OWNSYMBOLS, (CEdit&)m_SymbolsEdit);
+    DDX_Control(pDX, IDC_OWNSYMBOLS, (CEdit&)m_SymbolsEdit);
 
-  // Because we can show the generated password when used from Mangage->Generate
-  DDX_Control(pDX, IDC_PASSWORD, m_ex_password);
+    // Because we can show the generated password when used from Mangage->Generate
+    DDX_Control(pDX, IDC_PASSWORD, m_ex_password);
 
-  DDX_Control(pDX, IDC_POLICYNAME, m_PolicyNameEdit);
-  DDX_Control(pDX, IDC_POLICYLIST, m_cbxPolicyNames);
-  //}}AFX_DATA_MAP
+    DDX_Control(pDX, IDC_POLICYNAME, m_PolicyNameEdit);
+    DDX_Control(pDX, IDC_POLICYLIST, m_cbxPolicyNames);
+    //}}AFX_DATA_MAP
 }
 
 BEGIN_MESSAGE_MAP(CPasswordPolicyDlg, CPWDialog)
-  //{{AFX_MSG_MAP(CPasswordPolicyDlg)
-  ON_BN_CLICKED(IDOK, OnOK)
-  ON_BN_CLICKED(IDCANCEL, OnCancel)
-  ON_BN_CLICKED(ID_HELP, OnHelp)
+//{{AFX_MSG_MAP(CPasswordPolicyDlg)
+ON_BN_CLICKED(IDOK, OnOK)
+ON_BN_CLICKED(IDCANCEL, OnCancel)
+ON_BN_CLICKED(ID_HELP, OnHelp)
 
-  ON_BN_CLICKED(IDC_USEHEXDIGITS, OnUseHexdigits)
-  ON_BN_CLICKED(IDC_USELOWERCASE, OnUseLowerCase)
-  ON_BN_CLICKED(IDC_USEUPPERCASE, OnUseUpperCase)
-  ON_BN_CLICKED(IDC_USEDIGITS, OnUseDigits)
-  ON_BN_CLICKED(IDC_USESYMBOLS, OnUseSymbols)
-  ON_BN_CLICKED(IDC_EASYVISION, OnEasyVision)
-  ON_BN_CLICKED(IDC_PRONOUNCEABLE, OnMakePronounceable)
+ON_BN_CLICKED(IDC_USEHEXDIGITS, OnUseHexdigits)
+ON_BN_CLICKED(IDC_USELOWERCASE, OnUseLowerCase)
+ON_BN_CLICKED(IDC_USEUPPERCASE, OnUseUpperCase)
+ON_BN_CLICKED(IDC_USEDIGITS, OnUseDigits)
+ON_BN_CLICKED(IDC_USESYMBOLS, OnUseSymbols)
+ON_BN_CLICKED(IDC_EASYVISION, OnEasyVision)
+ON_BN_CLICKED(IDC_PRONOUNCEABLE, OnMakePronounceable)
 
-  // Because we can show the generated password when used from Mangage->Generate
-  ON_BN_CLICKED(IDC_GENERATEPASSWORD, OnGeneratePassword)
-  ON_BN_CLICKED(IDC_COPYPASSWORD, OnCopyPassword)
-  ON_EN_CHANGE(IDC_PASSWORD, OnENChangePassword)
+// Because we can show the generated password when used from Manage->Generate
+ON_BN_CLICKED(IDC_GENERATEPASSWORD, OnGeneratePassword)
+ON_BN_CLICKED(IDC_COPYPASSWORD, OnCopyPassword)
+ON_EN_CHANGE(IDC_PASSWORD, OnENChangePassword)
 
-  ON_BN_CLICKED(IDC_USEDEFAULTSYMBOLS, OnSymbols)
-  ON_BN_CLICKED(IDC_USEOWNSYMBOLS, OnSymbols)
+ON_BN_CLICKED(IDC_USENAMED_POLICY, OnUseNamedPolicy)
 
-  ON_BN_CLICKED(IDC_USENAMED_POLICY, OnUseNamedPolicy)
+ON_EN_KILLFOCUS(IDC_POLICYNAME, OnENChangePolicyName)
+ON_EN_KILLFOCUS(IDC_OWNSYMBOLS, OnENOwnSymbols)
 
-  ON_EN_KILLFOCUS(IDC_POLICYNAME, OnENChangePolicyName)
-  ON_EN_KILLFOCUS(IDC_OWNSYMBOLS, OnENOwnSymbols)
-
-  ON_CBN_SELCHANGE(IDC_POLICYLIST, OnNamesComboChanged)
-  //}}AFX_MSG_MAP
+ON_CBN_SELCHANGE(IDC_POLICYLIST, OnNamesComboChanged)
+//}}AFX_MSG_MAP
+ON_BN_CLICKED(IDC_RESET_SYMBOLS, &CPasswordPolicyDlg::OnSymbolReset)
 END_MESSAGE_MAP()
 
 /////////////////////////////////////////////////////////////////////////////
 // CPasswordPolicyDlg message handlers
 
+// Following copied from AddEdit_PasswordPolicy.cpp. Move to common mixin?
+static void setupBuddy(CWnd *p, int spinid, int id, int &length, short min = 0)
+{
+  CSpinButtonCtrl* pspin;
+
+  pspin = (CSpinButtonCtrl *)p->GetDlgItem(spinid);
+  pspin->SetBuddy(p->GetDlgItem(id));
+  pspin->SetRange(min, 1024);
+  pspin->SetBase(10);
+  pspin->SetPos((int)length);
+}
+
 BOOL CPasswordPolicyDlg::OnInitDialog()
 {
   CPWDialog::OnInitDialog();
 
-  // Verify ptr to DboxMain has been set up (call to SetPolicyData)
-  ASSERT(m_pDbx != NULL);
-  
   // If started with Tall and won't fit - return to be called again with Wide
-  if (m_bLongPPs && !m_pDbx->LongPPs(this)) {
+  if (m_bLongPPs && !GetMainDlg()->LongPPs(this)) {
     EndDialog(-1);
     return FALSE;
   }
@@ -197,27 +206,27 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
 
   CString cs_title;
   switch (m_uicaller) {
-    case IDS_OPTIONS:
-      // Set correct window title
-      cs_title.LoadString(IDS_EDIT_DEFAULT_POLICY);
-      SetWindowText(cs_title);
+  case IDS_OPTIONS:
+    // Set correct window title
+    cs_title.LoadString(IDS_EDIT_DEFAULT_POLICY);
+    SetWindowText(cs_title);
 
-      // These are only used in Manage -> Generate Password or Add/Edit Policy names
-      GetDlgItem(IDC_GENERATEPASSWORD)->EnableWindow(FALSE);
-      GetDlgItem(IDC_GENERATEPASSWORD)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_COPYPASSWORD)->EnableWindow(FALSE);
-      GetDlgItem(IDC_COPYPASSWORD)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_PASSWORD)->EnableWindow(FALSE);
-      GetDlgItem(IDC_PASSWORD)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_POLICYNAME)->EnableWindow(FALSE);
-      GetDlgItem(IDC_POLICYNAME)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_POLICYLIST)->EnableWindow(FALSE);
-      GetDlgItem(IDC_POLICYLIST)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_USENAMED_POLICY)->EnableWindow(FALSE);
-      GetDlgItem(IDC_USENAMED_POLICY)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_STATIC_NAMEDPOLICY)->ShowWindow(SW_HIDE);
-      break;
-    case IDS_GENERATEPASSWORD:
+    // These are only used in Manage -> Generate Password or Add/Edit Policy names
+    GetDlgItem(IDC_GENERATEPASSWORD)->EnableWindow(FALSE);
+    GetDlgItem(IDC_GENERATEPASSWORD)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_COPYPASSWORD)->EnableWindow(FALSE);
+    GetDlgItem(IDC_COPYPASSWORD)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_PASSWORD)->EnableWindow(FALSE);
+    GetDlgItem(IDC_PASSWORD)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_POLICYNAME)->EnableWindow(FALSE);
+    GetDlgItem(IDC_POLICYNAME)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_POLICYLIST)->EnableWindow(FALSE);
+    GetDlgItem(IDC_POLICYLIST)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_USENAMED_POLICY)->EnableWindow(FALSE);
+    GetDlgItem(IDC_USENAMED_POLICY)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_STATIC_NAMEDPOLICY)->ShowWindow(SW_HIDE);
+    break;
+  case IDS_GENERATEPASSWORD:
     {
       // Set correct window title
       cs_title.LoadString(IDS_GENERATEPASSWORD);
@@ -238,10 +247,10 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
 
       // Get all password policy names
       std::vector<std::wstring> vNames;
-      m_pDbx->GetPolicyNames(vNames);
+      GetMainDlg()->GetPolicyNames(vNames);
 
       // Add Default
-      CString cs_default(MAKEINTRESOURCE(IDSC_DEFAULT_POLICY));
+      const CString cs_default(MAKEINTRESOURCE(IDSC_DEFAULT_POLICY));
       m_cbxPolicyNames.AddString(cs_default);
 
       for (std::vector<std::wstring>::iterator iter = vNames.begin();
@@ -281,69 +290,58 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
 
       // Remove password character so that the password is displayed
       m_ex_password.SetPasswordChar(0);
+
+      // Load bitmap
+      UINT nImageID = PWSprefs::GetInstance()->GetPref(PWSprefs::UseNewToolbar) ?
+        IDB_COPYPASSWORD_NEW : IDB_COPYPASSWORD_CLASSIC;
+      BOOL brc = m_CopyPswdBitmap.Attach(::LoadImage(
+                                                     ::AfxFindResourceHandle(MAKEINTRESOURCE(nImageID), RT_BITMAP),
+                                                     MAKEINTRESOURCE(nImageID), IMAGE_BITMAP, 0, 0,
+                                                     (LR_DEFAULTSIZE | LR_CREATEDIBSECTION | LR_SHARED)));
+      ASSERT(brc);
+
+      FixBitmapBackground(m_CopyPswdBitmap);
+      CButton *pBtn = (CButton *)GetDlgItem(IDC_COPYPASSWORD);
+      pBtn->SetBitmap(m_CopyPswdBitmap);
+
       break;
     }
-    case IDS_PSWDPOLICY:
-      // Set correct window title
-      cs_title.LoadString(m_policyname.IsEmpty() ? IDS_ADD_NAMED_POLICY : IDS_EDIT_NAMED_POLICY);
-      SetWindowText(cs_title);
+  case IDS_PSWDPOLICY:
+    // Set correct window title
+    cs_title.LoadString(m_policyname.IsEmpty() ? IDS_ADD_NAMED_POLICY : IDS_EDIT_NAMED_POLICY);
+    SetWindowText(cs_title);
 
-      // These are only used in Manage -> Password Policy
-      GetDlgItem(IDC_GENERATEPASSWORD)->EnableWindow(FALSE);
-      GetDlgItem(IDC_GENERATEPASSWORD)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_COPYPASSWORD)->EnableWindow(FALSE);
-      GetDlgItem(IDC_COPYPASSWORD)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_PASSWORD)->EnableWindow(FALSE);
-      GetDlgItem(IDC_PASSWORD)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_POLICYLIST)->EnableWindow(FALSE);
-      GetDlgItem(IDC_POLICYLIST)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_USENAMED_POLICY)->EnableWindow(FALSE);
-      GetDlgItem(IDC_USENAMED_POLICY)->ShowWindow(SW_HIDE);
-      GetDlgItem(IDC_STATIC_NAMEDPOLICY)->ShowWindow(SW_SHOW);
+    // These are only used in Manage -> Password Policy
+    GetDlgItem(IDC_GENERATEPASSWORD)->EnableWindow(FALSE);
+    GetDlgItem(IDC_GENERATEPASSWORD)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_COPYPASSWORD)->EnableWindow(FALSE);
+    GetDlgItem(IDC_COPYPASSWORD)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_PASSWORD)->EnableWindow(FALSE);
+    GetDlgItem(IDC_PASSWORD)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_POLICYLIST)->EnableWindow(FALSE);
+    GetDlgItem(IDC_POLICYLIST)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_USENAMED_POLICY)->EnableWindow(FALSE);
+    GetDlgItem(IDC_USENAMED_POLICY)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_STATIC_NAMEDPOLICY)->ShowWindow(SW_SHOW);
 
-      if (!m_policyname.IsEmpty() && m_iter->second.usecount > 0) {
-        // Cannot edit the policy 'Name' if it is present and use count > 0
-        m_PolicyNameEdit.SetReadOnly(TRUE);
-        m_PolicyNameEdit.EnableWindow(FALSE);
-      }
+    if (!m_policyname.IsEmpty() && m_iter->second.usecount > 0) {
+      // Cannot edit the policy 'Name' if it is present and use count > 0
+      m_PolicyNameEdit.SetReadOnly(TRUE);
+      m_PolicyNameEdit.EnableWindow(FALSE);
+    }
 
-      m_PolicyNameEdit.SetWindowText(m_policyname);
-      // Max. length of policy name is 255 - only 2 hex digits used for length
-      // in database header
-      m_PolicyNameEdit.SetLimitText(255);
-      break;
+    m_PolicyNameEdit.SetWindowText(m_policyname);
+    // Max. length of policy name is 255 - only 2 hex digits used for length
+    // in database header
+    m_PolicyNameEdit.SetLimitText(255);
+    break;
   }
 
-  CSpinButtonCtrl *pspin  = (CSpinButtonCtrl *)GetDlgItem(IDC_PWLENSPIN);
-  CSpinButtonCtrl *pspinD = (CSpinButtonCtrl *)GetDlgItem(IDC_SPINDIGITS);
-  CSpinButtonCtrl *pspinL = (CSpinButtonCtrl *)GetDlgItem(IDC_SPINLOWERCASE);
-  CSpinButtonCtrl *pspinS = (CSpinButtonCtrl *)GetDlgItem(IDC_SPINSYMBOLS);
-  CSpinButtonCtrl *pspinU = (CSpinButtonCtrl *)GetDlgItem(IDC_SPINUPPERCASE);
-
-  pspin->SetBuddy(GetDlgItem(IDC_DEFPWLENGTH));
-  pspin->SetRange(4, 1024);
-  pspin->SetBase(10);
-  pspin->SetPos(m_PWDefaultLength);
-
-  pspinD->SetBuddy(GetDlgItem(IDC_MINDIGITLENGTH));
-  pspinD->SetRange(0, 1024);
-  pspinD->SetBase(10);
-  pspinD->SetPos(m_PWDigitMinLength);
-
-  pspinL->SetBuddy(GetDlgItem(IDC_MINLOWERLENGTH));
-  pspinL->SetRange(0, 1024);
-  pspinL->SetBase(10);
-  pspinL->SetPos(m_PWLowerMinLength);
-
-  pspinS->SetBuddy(GetDlgItem(IDC_MINSYMBOLLENGTH));
-  pspinS->SetRange(0, 1024);
-  pspinS->SetBase(10);
-  pspinS->SetPos(m_PWSymbolMinLength);
-
-  pspinU->SetBuddy(GetDlgItem(IDC_MINUPPERLENGTH));
-  pspinU->SetRange(0, 1024);
-  pspinU->SetBase(10);
-  pspinU->SetPos(m_PWUpperMinLength);
+  setupBuddy(this, IDC_PWLENSPIN, IDC_DEFPWLENGTH, m_PWDefaultLength, 4);
+  setupBuddy(this, IDC_SPINDIGITS, IDC_MINDIGITLENGTH, m_PWDigitMinLength);
+  setupBuddy(this, IDC_SPINLOWERCASE, IDC_MINLOWERLENGTH, m_PWLowerMinLength);
+  setupBuddy(this, IDC_SPINUPPERCASE, IDC_MINUPPERLENGTH, m_PWUpperMinLength);
+  setupBuddy(this, IDC_SPINSYMBOLS, IDC_MINSYMBOLLENGTH, m_PWSymbolMinLength);
 
   m_save[SAVE_LOWERCASE] = m_PWUseLowercase;
   m_save[SAVE_UPPERCASE] = m_PWUseUppercase;
@@ -373,15 +371,14 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
   if (m_PWEasyVision == TRUE)
     iSet = EVPR_EV;
   else
-  if (m_PWMakePronounceable == TRUE)
-    iSet = EVPR_PR;
+    if (m_PWMakePronounceable == TRUE)
+      iSet = EVPR_PR;
   do_easyorpronounceable(iSet);
 
-  m_SymbolsEdit.SetWindowText(m_Symbols);
+  GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(m_PWUseSymbols);
+  GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(m_PWUseSymbols);
 
-  GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(m_PWUseSymbols);
-  GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(m_PWUseSymbols);
-  GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow((m_PWUseSymbols == TRUE && m_UseOwnSymbols == OWN_SYMBOLS) ? TRUE : FALSE);
+  m_SymbolsEdit.SetWindowText(m_Symbols);
 
   if (m_uicaller == IDS_GENERATEPASSWORD) {
     // Disable Specific policy controls as default is to use a named policy (database default)
@@ -445,9 +442,7 @@ void CPasswordPolicyDlg::OnCancel()
       m_PWEasyVision        != m_oldPWEasyVision        ||
       m_PWUseHexdigits      != m_oldPWUseHexdigits      ||
       m_PWMakePronounceable != m_oldPWMakePronounceable ||
-      m_UseOwnSymbols       != m_oldUseOwnSymbols       ||
-      (m_UseOwnSymbols      == OWN_SYMBOLS &&
-       m_Symbols            != m_oldSymbols)) {
+      m_Symbols            != m_oldSymbols) {
     // Confirm Cancel from User
     CGeneralMsgBox gmb;
     if (gmb.AfxMessageBox(IDS_AREYOUSURE_OPT,
@@ -461,22 +456,15 @@ exit:
 
 void CPasswordPolicyDlg::OnHelp()
 {
-  CString cs_HelpTopic;
-  cs_HelpTopic = app.GetHelpFileName() + L"::/html/password_policies.html";
-  HtmlHelp(DWORD_PTR((LPCWSTR)cs_HelpTopic), HH_DISPLAY_TOPIC);
+  ShowHelp(L"::/html/password_policies.html");
 }
 
 void CPasswordPolicyDlg::SetPolicyData(CString &cs_policyname,
-                                       PSWDPolicyMap &MapPSWDPLC,
-                                       DboxMain *pDbx)
+                                       PSWDPolicyMap &MapPSWDPLC)
 {
   m_MapPSWDPLC = MapPSWDPLC;
   m_policyname = m_oldpolicyname = cs_policyname;
 
-  // Save ptr to DboxMain
-  ASSERT(pDbx != NULL);
-  m_pDbx = pDbx;
-
   // Only called for Password Policies
   if (m_uicaller != IDS_PSWDPOLICY)
     return;
@@ -520,7 +508,6 @@ void CPasswordPolicyDlg::SetPolicyData(CString &cs_policyname,
 
   CString cs_symbols = xst_pp.symbols.c_str();
   m_Symbols = m_oldSymbols = cs_symbols;
-  m_UseOwnSymbols = m_oldUseOwnSymbols = cs_symbols.IsEmpty() ? DEFAULT_SYMBOLS : OWN_SYMBOLS;
 }
 
 void CPasswordPolicyDlg::OnNamesComboChanged()
@@ -567,13 +554,6 @@ void CPasswordPolicyDlg::OnNamesComboChanged()
 
   m_Symbols = m_oldSymbols = xst_pp.symbols.c_str();
   m_SymbolsEdit.SetWindowText(m_Symbols);
-  if (m_Symbols.IsEmpty()) {
-    m_UseOwnSymbols = DEFAULT_SYMBOLS;
-    ((CButton *)GetDlgItem(IDC_USEDEFAULTSYMBOLS))->SetCheck(BST_CHECKED);
-  } else {
-    m_UseOwnSymbols = OWN_SYMBOLS;
-    ((CButton *)GetDlgItem(IDC_USEOWNSYMBOLS))->SetCheck(BST_CHECKED);
-  }
 
   UpdateData(FALSE);
 }
@@ -603,10 +583,8 @@ void CPasswordPolicyDlg::do_hex(const bool bHex)
       GetDlgItem(LenTxts[i * 2 + 1])->EnableWindow(FALSE);
     }
 
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(FALSE);
     GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(FALSE);
+    GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(FALSE);
 
     m_savelen[SAVE_LOWERCASE] = m_PWLowerMinLength;
     m_savelen[SAVE_UPPERCASE] = m_PWUpperMinLength;
@@ -624,7 +602,7 @@ void CPasswordPolicyDlg::do_hex(const bool bHex)
 
     for (i = 0; i < N_HEX_LENGTHS; i++) {
       UINT id = nonHexLengths[i];
-      cs_value.Format(L"%d", m_savelen[i]);
+      cs_value.Format(L"%u", m_savelen[i]);
       GetDlgItem(id)->SetWindowText(cs_value);
       GetDlgItem(id)->EnableWindow(m_save[i]);
       GetDlgItem(nonHexLengthSpins[i])->EnableWindow(m_save[i]);
@@ -635,12 +613,8 @@ void CPasswordPolicyDlg::do_hex(const bool bHex)
     BOOL bEnable = (IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED &&
                     m_PWEasyVision == FALSE && m_PWMakePronounceable == FALSE) ?
                       TRUE : FALSE;
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow((bEnable == TRUE &&
-                                              m_UseOwnSymbols == OWN_SYMBOLS) ?
-                                              TRUE : FALSE);
+    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(bEnable);
+    GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(bEnable);
 
     m_PWLowerMinLength = m_savelen[SAVE_LOWERCASE];
     m_PWUpperMinLength = m_savelen[SAVE_UPPERCASE];
@@ -670,12 +644,6 @@ void CPasswordPolicyDlg::do_easyorpronounceable(const int iSet)
       GetDlgItem(LenTxts[2 * i + 1])->ShowWindow(SW_HIDE);
     }
 
-    GetDlgItem(IDC_USESYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(FALSE);
-
     m_savelen[SAVE_LOWERCASE] = m_PWLowerMinLength;
     m_savelen[SAVE_UPPERCASE] = m_PWUpperMinLength;
     m_savelen[SAVE_DIGITS] = m_PWDigitMinLength;
@@ -691,92 +659,65 @@ void CPasswordPolicyDlg::do_easyorpronounceable(const int iSet)
 
     BOOL bEnable = (IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED) ? TRUE : FALSE;
     GetDlgItem(IDC_USESYMBOLS)->EnableWindow(TRUE);
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(bEnable);
-    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow((bEnable == TRUE && m_UseOwnSymbols == OWN_SYMBOLS) ?
-                                             TRUE : FALSE);
+    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(bEnable);
+    GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(bEnable);
 
     m_PWLowerMinLength = m_savelen[SAVE_LOWERCASE];
     m_PWUpperMinLength = m_savelen[SAVE_UPPERCASE];
     m_PWDigitMinLength = m_savelen[SAVE_DIGITS];
     m_PWSymbolMinLength = m_savelen[SAVE_SYMBOLS];
   }
-  
-  stringT *pst_symbols;
-  if (m_PWEasyVision == TRUE)
-    pst_symbols = &m_easyvision_symbols;
-  else if (m_PWMakePronounceable == TRUE)
-    pst_symbols = &m_pronounceable_symbols;
-  else
-    pst_symbols = &m_std_symbols;
+}
 
-  GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->SetWindowText((*pst_symbols).c_str());
+// Following from AddEdit_PasswordPolicy - TBD - move to common mixin class
+void CPasswordPolicyDlg::do_useX(UseX x, int &minlength)
+{
+  const struct { int cb; int edit; int spin; int left; int right;} controls[] = {
+    {IDC_USELOWERCASE, IDC_MINLOWERLENGTH,  IDC_SPINLOWERCASE, IDC_STATIC_LC1, IDC_STATIC_LC2},
+    {IDC_USEUPPERCASE, IDC_MINUPPERLENGTH,  IDC_SPINUPPERCASE, IDC_STATIC_UC1, IDC_STATIC_UC2},
+    {IDC_USEDIGITS,    IDC_MINDIGITLENGTH,  IDC_SPINDIGITS,    IDC_STATIC_DG1, IDC_STATIC_DG2},
+    {IDC_USESYMBOLS,   IDC_MINSYMBOLLENGTH, IDC_SPINSYMBOLS,   IDC_STATIC_SY1, IDC_STATIC_SY2},
+  };
+  UnselectNamedPolicy();
+  UpdateData(TRUE);
+  BOOL bChecked = (IsDlgButtonChecked(controls[x].cb) == BST_CHECKED) ? TRUE : FALSE;
+
+  GetDlgItem(controls[x].edit)->EnableWindow(bChecked);
+  GetDlgItem(controls[x].spin)->EnableWindow(bChecked);
+  GetDlgItem(controls[x].left)->EnableWindow(bChecked);
+  GetDlgItem(controls[x].right)->EnableWindow(bChecked);
+  if (x == USESYM) {
+    GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(bChecked);
+    GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(bChecked);
+  }
+  minlength = bChecked;  // Based on FALSE=0 & TRUE=1
+  UpdateData(FALSE);
 }
 
 void CPasswordPolicyDlg::OnUseLowerCase()
 {
-  UpdateData(TRUE);
-  BOOL bChecked = (IsDlgButtonChecked(IDC_USELOWERCASE) == BST_CHECKED) ? TRUE : FALSE;
-
-  GetDlgItem(IDC_MINLOWERLENGTH)->EnableWindow(bChecked);
-  GetDlgItem(IDC_SPINLOWERCASE)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_LC1)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_LC2)->EnableWindow(bChecked);
-  m_PWLowerMinLength = bChecked;  // Based on FALSE=0 & TRUE=1
-  UpdateData(FALSE);
+  do_useX(USELOWER, m_PWLowerMinLength);
 }
 
 void CPasswordPolicyDlg::OnUseUpperCase()
 {
-  UpdateData(TRUE);
-  BOOL bChecked = (IsDlgButtonChecked(IDC_USEUPPERCASE) == BST_CHECKED) ? TRUE : FALSE;
-
-  GetDlgItem(IDC_MINUPPERLENGTH)->EnableWindow(bChecked);
-  GetDlgItem(IDC_SPINUPPERCASE)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_UC1)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_UC2)->EnableWindow(bChecked);
-  m_PWUpperMinLength = bChecked;  // Based on FALSE=0 & TRUE=1
-  UpdateData(FALSE);
+  do_useX(USEUPPER, m_PWUpperMinLength);
 }
 
 void CPasswordPolicyDlg::OnUseDigits()
 {
-  UpdateData(TRUE);
-  BOOL bChecked = (IsDlgButtonChecked(IDC_USEDIGITS) == BST_CHECKED) ? TRUE : FALSE;
-
-  GetDlgItem(IDC_MINDIGITLENGTH)->EnableWindow(bChecked);
-  GetDlgItem(IDC_SPINDIGITS)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_DG1)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_DG2)->EnableWindow(bChecked);
-  m_PWDigitMinLength = bChecked;  // Based on FALSE=0 & TRUE=1
-  UpdateData(FALSE);
+  do_useX(USEDIGITS, m_PWDigitMinLength);
 }
 
 void CPasswordPolicyDlg::OnUseSymbols()
 {
-  UpdateData(TRUE);
-
-  BOOL bChecked = (IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED &&
-                   m_PWEasyVision == FALSE && m_PWMakePronounceable == FALSE) ? TRUE : FALSE;
-
-  GetDlgItem(IDC_MINSYMBOLLENGTH)->EnableWindow(bChecked);
-  GetDlgItem(IDC_SPINSYMBOLS)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_SY1)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_SY2)->EnableWindow(bChecked);
-
-  GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(bChecked);
-  GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(bChecked);
-  GetDlgItem(IDC_STATIC_DEFAULTSYMBOLS)->EnableWindow(bChecked);
-  GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow((bChecked == TRUE && m_UseOwnSymbols == OWN_SYMBOLS) ? TRUE : FALSE);
-
-  m_PWSymbolMinLength = bChecked;  // Based on FALSE=0 & TRUE=1
-
+  do_useX(USESYM, m_PWSymbolMinLength);
   UpdateData(FALSE);
 }
 
 void CPasswordPolicyDlg::OnUseHexdigits()
 {
+  UnselectNamedPolicy();
   UpdateData(TRUE);
 
   do_hex(IsDlgButtonChecked(IDC_USEHEXDIGITS) == BST_CHECKED);
@@ -786,6 +727,7 @@ void CPasswordPolicyDlg::OnUseHexdigits()
 
 void CPasswordPolicyDlg::OnEasyVision()
 {
+  UnselectNamedPolicy();
   UpdateData(TRUE);
 
   if (m_PWEasyVision && m_PWMakePronounceable) {
@@ -798,13 +740,15 @@ void CPasswordPolicyDlg::OnEasyVision()
 
   const bool bChecked = (IsDlgButtonChecked(IDC_EASYVISION) == BST_CHECKED);
 
-  do_easyorpronounceable(bChecked ? EVPR_EV : EVPR_NONE);
+  do_easyorpronounceable(bChecked ? EVPR_EV : EVPR_NONE);  
+  do_reset_symbols(false);
   // Do not use UpdateData(FALSE) here or
   // all the good work in "do_easyorpronounceable" will be undone
 }
 
 void CPasswordPolicyDlg::OnMakePronounceable()
 {
+  UnselectNamedPolicy();
   UpdateData(TRUE);
 
   if (m_PWEasyVision && m_PWMakePronounceable) {
@@ -825,21 +769,12 @@ void CPasswordPolicyDlg::OnMakePronounceable()
     return;
   }
 
-  do_easyorpronounceable(bChecked ? EVPR_PR : EVPR_NONE);
+  do_easyorpronounceable(bChecked ? EVPR_PR : EVPR_NONE);  
+  do_reset_symbols(false);
   // Do not use UpdateData(FALSE) here or
   // all the good work in "do_easyorpronounceable" will be undone
 }
 
-void CPasswordPolicyDlg::OnSymbols()
-{
-  m_UseOwnSymbols = ((CButton *)GetDlgItem(IDC_USEDEFAULTSYMBOLS))->GetCheck() == BST_CHECKED ?
-                        DEFAULT_SYMBOLS : OWN_SYMBOLS;
-
-  GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(m_UseOwnSymbols == DEFAULT_SYMBOLS ? FALSE : TRUE);
-  if (m_UseOwnSymbols == OWN_SYMBOLS)
-    GetDlgItem(IDC_OWNSYMBOLS)->SetFocus();
-}
-
 BOOL CPasswordPolicyDlg::Validate()
 {
   CGeneralMsgBox gmb;
@@ -908,8 +843,6 @@ void CPasswordPolicyDlg::OnGeneratePassword()
 {
   UpdateData(TRUE);
 
-  stringT st_symbols;
-
   PWPolicy st_pp;
 
   if (m_UseNamedPolicy == BST_UNCHECKED) {
@@ -937,13 +870,8 @@ void CPasswordPolicyDlg::OnGeneratePassword()
     st_pp.symbolminlength = m_PWSymbolMinLength;
     st_pp.upperminlength = m_PWUpperMinLength;
 
-    if (m_UseOwnSymbols == OWN_SYMBOLS) {
-      m_SymbolsEdit.GetWindowText(m_Symbols);
-      st_symbols = LPCWSTR(m_Symbols);
-    } else
-      CPasswordCharPool::GetDefaultSymbols(st_symbols);
-
-    st_pp.symbols = st_symbols.c_str();
+    m_SymbolsEdit.GetWindowText(m_Symbols);
+    st_pp.symbols = LPCWSTR(m_Symbols);
   } else {
     // Use named policy
     if (m_PolicyNameEdit.IsWindowVisible()) {
@@ -953,16 +881,11 @@ void CPasswordPolicyDlg::OnGeneratePassword()
       m_cbxPolicyNames.GetLBText(index, m_policyname);
     }
     StringX sxPolicyName(m_policyname);
-    m_pDbx->GetPolicyFromName(sxPolicyName, st_pp);
-
-    if (st_pp.symbols.empty()) {
-      CPasswordCharPool::GetDefaultSymbols(st_symbols);
-      st_pp.symbols = st_symbols.c_str();
-    }
+    GetMainDlg()->GetPolicyFromName(sxPolicyName, st_pp);
   }
 
   StringX passwd;
-  m_pDbx->MakeRandomPassword(passwd, st_pp);
+  GetMainDlg()->MakeRandomPassword(passwd, st_pp);
   m_password = passwd.c_str();
   m_ex_password.SetWindowText(m_password);
   m_ex_password.Invalidate();
@@ -974,8 +897,8 @@ void CPasswordPolicyDlg::OnCopyPassword()
 {
   UpdateData(TRUE);
 
-  m_pDbx->SetClipboardData(m_password);
-  m_pDbx->UpdateLastClipboardAction(CItemData::PASSWORD);
+  GetMainDlg()->SetClipboardData(m_password);
+  GetMainDlg()->UpdateLastClipboardAction(CItemData::PASSWORD);
 }
 
 void CPasswordPolicyDlg::OnENChangePassword()
@@ -1007,8 +930,6 @@ void CPasswordPolicyDlg::OnENOwnSymbols()
 
     // Select Default symbols - then uncheck use symbols
     ((CButton *)GetDlgItem(IDC_USESYMBOLS))->SetCheck(BST_UNCHECKED);
-    ((CButton *)GetDlgItem(IDC_USEDEFAULTSYMBOLS))->SetCheck(BST_CHECKED);
-    ((CButton *)GetDlgItem(IDC_USEOWNSYMBOLS))->SetCheck(BST_UNCHECKED);
 
     // Setup variables etc.
     OnUseSymbols();
@@ -1051,13 +972,7 @@ void CPasswordPolicyDlg::SetSpecificPolicyControls(const BOOL bEnable)
       GetDlgItem(nonHexLengthSpins[i])->EnableWindow(FALSE);
     }
 
-    // Disable lengths
-    GetDlgItem(IDC_STATIC_PSWDLENGTH)->EnableWindow(FALSE);
-    GetDlgItem(IDC_DEFPWLENGTH)->EnableWindow(FALSE);
-
     // Disable Symbols
-    GetDlgItem(IDC_USEDEFAULTSYMBOLS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_USEOWNSYMBOLS)->EnableWindow(FALSE);
     m_SymbolsEdit.EnableWindow(FALSE);
   } else {
     // Restore previous values
@@ -1123,8 +1038,7 @@ bool CPasswordPolicyDlg::UpdatePasswordPolicy()
   st_pp.symbolminlength = m_PWSymbolMinLength;
   st_pp.upperminlength = m_PWUpperMinLength;
 
-  st_pp.symbols = (m_PWUseSymbols == TRUE && m_UseOwnSymbols == OWN_SYMBOLS) ? 
-    m_Symbols : L"";
+  st_pp.symbols = m_Symbols;
 
   if (m_uicaller == IDS_OPTIONS) {
     // Update database default
@@ -1152,3 +1066,33 @@ bool CPasswordPolicyDlg::UpdatePasswordPolicy()
   }
   return true;
 }
+
+void CPasswordPolicyDlg::UnselectNamedPolicy()
+{
+  ((CButton *)GetDlgItem(IDC_USENAMED_POLICY))->SetCheck(BST_UNCHECKED);
+  m_cbxPolicyNames.EnableWindow(FALSE);
+}
+
+
+void CPasswordPolicyDlg::OnSymbolReset()
+{
+  do_reset_symbols(true);
+}
+
+void CPasswordPolicyDlg::do_reset_symbols(bool restore_defaults)
+{
+  stringT st_symbols;
+  if (m_PWEasyVision == TRUE)
+    st_symbols = CPasswordCharPool::GetEasyVisionSymbols();
+  else if (m_PWMakePronounceable == TRUE)
+    st_symbols = CPasswordCharPool::GetPronounceableSymbols();
+  else {
+    if (restore_defaults)
+      CPasswordCharPool::ResetDefaultSymbols(); // restore the preference!
+    st_symbols = CPasswordCharPool::GetDefaultSymbols();
+  }
+
+  m_SymbolsEdit.SetValidSym(st_symbols);
+  m_SymbolsEdit.SetWindowText(st_symbols.c_str());
+  m_Symbols = st_symbols.c_str();
+}
