@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -17,14 +17,9 @@
 
 #include "PasswordSafe.h"
 
-#if defined(POCKET_PC)
-#include "pocketpc/PocketPC.h"
-#include "pocketpc/resource.h"
-#else
 #include "resource.h"
 #include "resource2.h"  // Menu, Toolbar & Accelerator resources
 #include "resource3.h"  // String resources
-#endif
 
 #include "ThisMfcApp.h"
 #include "DboxMain.h"
@@ -49,12 +44,14 @@
 #include "os/dir.h"
 #include "os/file.h"
 #include "os/env.h"
+#include "os/lib.h"
 
 #include "Shlwapi.h"
 
 #include <vector>
 #include <errno.h>
 #include <io.h>
+#include <ddeml.h>
 
 // Only produce minidumps in release code
 #ifndef _DEBUG
@@ -74,6 +71,18 @@ static char THIS_FILE[] = __FILE__;
 const UINT ThisMfcApp::m_uiRegMsg   = RegisterWindowMessage(UNIQUE_PWS_GUID);
 const UINT ThisMfcApp::m_uiWH_SHELL = RegisterWindowMessage(UNIQUE_PWS_SHELL);
 
+// Message Filter processing (WH_MSGFILTER)  [_AfxMsgFilterHook from thrdcore.cpp]
+LRESULT CALLBACK MsgFilter(int code, WPARAM wParam, LPARAM lParam)
+{
+  CWinThread *pThread;
+  if (afxContextIsDLL || (code < 0 && code != MSGF_DDEMGR) ||
+    (pThread = AfxGetThread()) == NULL) {
+    return ::CallNextHookEx(_afxThreadState->m_hHookOldMsgFilter, code, wParam, lParam);
+  }
+  ASSERT(pThread != NULL);
+  return (LRESULT)pThread->ProcessMessageFilter(code, (LPMSG)lParam);
+}
+
 BEGIN_MESSAGE_MAP(ThisMfcApp, CWinApp)
 END_MESSAGE_MAP()
 
@@ -88,11 +97,7 @@ extern wchar_t *wcCaption;
 #endif
 
 ThisMfcApp::ThisMfcApp() :
-#if defined(POCKET_PC)
-  m_bUseAccelerator(false),
-#else
   m_bUseAccelerator(true),
-#endif
   m_pMRU(NULL), m_TrayLockedState(LOCKED), m_pTrayIcon(NULL),
   m_HotKeyPressed(false), m_hMutexOneInstance(NULL),
   m_ghAccelTable(NULL), m_pMainMenu(NULL),
@@ -136,36 +141,15 @@ ThisMfcApp::ThisMfcApp() :
   csRevision.ReleaseBuffer();
 #endif
 
-  // {kjp} Temporary until I'm sure that PwsPlatform.h configures the endianness properly
-#if defined(POCKET_PC)
-  // Double check that *_ENDIAN has been correctly set!
-#if defined(PWS_LITTLE_ENDIAN)
-  unsigned char buf[4] = { 1, 0, 0, 0 };
-  unsigned int ii = 1;
-#define ENDIANNESS1 0 // Little
-#define ENDIANNESS2 1 // Big
-#elif defined(PWS_BIG_ENDIAN)
-  unsigned char buf[4] = { 0, 0, 0, 1 };
-  unsigned int ii = 1;
-#define ENDIANNESS1 1 // Big
-#define ENDIANNESS2 0 // Little
-#endif
-  if (*(unsigned int*)buf != ii) {
-    CGeneralMsgBox gmb;
-    CString cs_msg, cs_e1, cs_e2;
-    cs_e1.LoadString(ENDIANNESS1 == 0 ? IDS_LITTLEENDIAN : IDS_BIGENDIAN);
-    cs_e2.LoadString(ENDIANNESS2 == 0 ? IDS_LITTLEENDIAN : IDS_BIGENDIAN);
-    cs_msg.Format(IDS_ENDIANERROR, cs_e1, cs_e2);
-    gmb.AfxMessageBox(cs_msg);
-  }
-#endif
   // Set this process to be one of the first to be shut down:
   SetProcessShutdownParameters(0x3ff, 0);
   PWSprefs::SetReporter(&aReporter);
   PWScore::SetReporter(&aReporter);
   PWScore::SetAsker(&anAsker);
   EnableHtmlHelp();
-  CoInitialize(NULL); // Initializes the COM library (for XML processing)
+  CoInitializeEx(NULL, COINIT_APARTMENTTHREADED); // Initializes the COM library
+  //                                                 (for XML and Yubikeyprocessing)
+  AfxEnableControlContainer();
   AfxOleInit();
 }
 
@@ -196,23 +180,19 @@ ThisMfcApp::~ThisMfcApp()
   PWSprefs::DeleteInstance();
   PWSrand::DeleteInstance();
   PWSversion::DeleteInstance();
-  Fonts::DeleteInstance();
+  Fonts::GetInstance()->DeleteInstance();
   PWSLog::DeleteLog();
 
   CoUninitialize(); // Uninitialize COM library
 
-#if !defined(POCKET_PC)
   // WinApp::HtmlHelp asserts that main windows is valid, which (1) isn't true
   // here, and (2) is irrelevant for HH_CLOSE_ALL, so we call ::HtmlHelp
   ::HtmlHelp(NULL, NULL, HH_CLOSE_ALL, 0);
-#endif
-
 #ifndef _DEBUG
   RemoveFaultHandler();
 #endif
 }
 
-#if !defined(POCKET_PC)
 static void Usage()
 {
   CGeneralMsgBox gmb;
@@ -239,7 +219,6 @@ static bool CheckFile(const CString &fn)
     return false;
   }
 }
-#endif // !POCKET_PC
 
 void ThisMfcApp::SetMinidumpUserStreams(const bool bOpen, const bool bRW, UserStream iStream)
 {
@@ -257,8 +236,8 @@ void ThisMfcApp::SetMinidumpUserStreams(const bool bOpen, const bool bRW, UserSt
 
 int ThisMfcApp::ExitInstance()
 {
-  if (m_hInstResDLL != NULL && m_hInstResDLL != m_hInstance)
-    FreeLibrary(m_hInstResDLL);
+  if (m_hInstResDLL != m_hInstance)
+    pws_os::FreeLibrary(m_hInstResDLL);
 
   CWinApp::ExitInstance();
   return 0;
@@ -367,8 +346,7 @@ void ThisMfcApp::LoadLocalizedStuff()
 
   if (cs_LANG == L"EN" && cs_CTRY.IsEmpty()) {
     // This means use the embedded resources!
-    if (m_hInstResDLL)
-      FreeLibrary(m_hInstResDLL);
+    pws_os::FreeLibrary(m_hInstResDLL);
     m_hInstResDLL = NULL;
     pws_os::Trace(L"Using embedded resources\n");
 
@@ -406,14 +384,16 @@ void ThisMfcApp::LoadLocalizedStuff()
 
   // Find reseource-only DLL if requested
   const CString format_string = (cs_CTRY.IsEmpty()) ?
-                      L"%spwsafe%s%s.dll" : L"%spwsafe%s_%s.dll";
-  cs_ResPath.Format(format_string, cs_ExePath, cs_LANG, cs_CTRY);
-  m_hInstResDLL = LoadLibrary(cs_ResPath);
+                      _T("pwsafe%s%s.dll") : _T("pwsafe%s_%s.dll");
+  cs_ResPath.Format(format_string, cs_LANG, cs_CTRY);
+  m_hInstResDLL = HMODULE(pws_os::LoadLibrary(LPCTSTR(cs_ResPath),
+                                              pws_os::LOAD_LIBRARY_APP));
 
   if (m_hInstResDLL == NULL && !cs_CTRY.IsEmpty()) {
     // Now try base
-    cs_ResPath.Format(L"%spwsafe%s.dll", cs_ExePath, cs_LANG);
-    m_hInstResDLL = LoadLibrary(cs_ResPath);
+    cs_ResPath.Format(_T("pwsafe%s.dll"), cs_LANG);
+    m_hInstResDLL = HMODULE(pws_os::LoadLibrary(LPCTSTR(cs_ResPath),
+                                                pws_os::LOAD_LIBRARY_APP));
   }
 
   if (m_hInstResDLL == NULL) {
@@ -430,7 +410,7 @@ void ThisMfcApp::LoadLocalizedStuff()
                   HIWORD(dw_fileMajorMinor), LOWORD(dw_fileMajorMinor), HIWORD(dw_fileBuildRevision),
                   HIWORD(m_dwMajorMinor), LOWORD(m_dwMajorMinor), HIWORD(m_dwBuildRevision));
       gmb.AfxMessageBox(oops);
-      FreeLibrary(m_hInstResDLL);
+      pws_os::FreeLibrary(m_hInstResDLL);
       m_hInstResDLL = NULL;
     } else { // Passed version check
       pws_os::Trace(L"Using language DLL '%s'.\n", cs_ResPath);
@@ -468,16 +448,10 @@ void ThisMfcApp::LoadLocalizedStuff()
   if (!helpFileFound && m_pszHelpFilePath != NULL && _tcslen(m_pszHelpFilePath) > 0) { // last resort
     wchar_t fname[_MAX_FNAME];
     wchar_t ext[_MAX_EXT];
-#if (_MSC_VER >= 1400)
     _wsplitpath_s(m_pszHelpFilePath, NULL, 0, NULL, 0, fname,
                   _MAX_FNAME, ext, _MAX_EXT);
     _wcslwr_s(fname, _MAX_FNAME);
     _wcslwr_s(ext, _MAX_EXT);
-#else
-    _wsplitpath(m_pszHelpFilePath, NULL, NULL, fname, ext);
-    _wcslwr(ext);
-    _wcslwr(fname);
-#endif
     cs_HelpPath.Format(L"%s%s", fname, ext);
   }
 
@@ -613,6 +587,14 @@ void ThisMfcApp::SetupMenu()
   minfo.dwMenuData = ID_EDITMENU;
   pMenu1->SetMenuInfo(&minfo);
 
+  // Do Edit Menu Export Entry submenu
+  pos2 = app.FindMenuItem(pMenu1, ID_EXPORTENTMENU);
+  ASSERT(pos2 != -1);
+
+  pMenu2 = pMenu1->GetSubMenu(pos2);
+  minfo.dwMenuData = ID_EXPORTENTMENU;
+  pMenu2->SetMenuInfo(&minfo);
+
   // Do View Menu
   pos1 = app.FindMenuItem(m_pMainMenu, ID_VIEWMENU);
   ASSERT(pos1 != -1);
@@ -676,7 +658,7 @@ void ThisMfcApp::SetupMenu()
 void ThisMfcApp::SetLanguage()
 {
   // Free old resource only DLL
-  if (m_hInstResDLL != NULL && m_hInstResDLL != m_hInstance)
+  if (m_hInstResDLL != m_hInstance)
     FreeLibrary(m_hInstResDLL);
 
   m_hInstResDLL = NULL;
@@ -711,11 +693,9 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
    * and one for personal use, and to set up a different shortcut for each.
    *
    * I think I'll keep the old functionality, but activate it with a "-e" or "-d" flag. (ronys)
-   * {kjp} ... and I've removed all of it from the Pocket PC build.
    */
 
   allDone = false;
-#if !defined(POCKET_PC)
   if (m_lpCmdLine[0] != L'\0') {
     CString args = m_lpCmdLine;
 
@@ -797,7 +777,7 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
           }
           // get password from user
           StringX passkey;
-          CCryptKeyEntry dlg(NULL);
+          CCryptKeyEntry dlg(isEncrypt);
           INT_PTR nResponse = dlg.DoModal();
 
           if (nResponse == IDOK) {
@@ -836,6 +816,9 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
           break;
         case L'R': case L'r':
           m_core.SetReadOnly(true);
+          /* Set initially RO too to prevent possible RO/RW mode change issues on open,
+            it will be set to real open state value later (in DboxMain::PostOpenProcessing) */
+          dbox.SetDBInitiallyRO(true);
           break;
         case L'S': case L's':
           startSilent = true;
@@ -896,7 +879,11 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
       } else { // arg isn't a flag, treat it as a filename
         if (CheckFile(*arg)) {
           fileGiven = true;
-          m_core.SetCurFile(arg->GetString());
+          // We send core the full path 'cause otherwise incr. backups & such get confused.
+          stringT abspath = pws_os::fullpath(arg->GetString());
+          if (abspath.empty()) // Should never happen, but try to slog on if so.
+            abspath = arg->GetString();
+          m_core.SetCurFile(abspath.c_str());
         } else {
           return false;
         }
@@ -907,7 +894,6 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
     if (startSilent && !fileGiven)
       dbox.SetStartClosed(true);
   } // Command line not empty
-#endif
   return true;
 }
 
@@ -1030,10 +1016,6 @@ BOOL ThisMfcApp::InitInstance()
   // PWScore needs it to get into database header if/when saved
   m_core.SetApplicationNameAndVersion(AfxGetAppName(), m_dwMajorMinor);
 
-#if defined(POCKET_PC)
-  SHInitExtraControls();
-#endif
-
   if (m_core.GetCurFile().empty()) {
     stringT path = prefs->GetPref(PWSprefs::CurrentFile).c_str();
     pws_os::AddDrive(path);
@@ -1053,22 +1035,19 @@ BOOL ThisMfcApp::InitInstance()
   The app object (here) should instead do the initial PasskeyEntry,
   and, if successful, move on to DboxMain.  I think. {jpr}
   */
-  m_pMainDlg = &dbox;
-  m_pMainWnd = m_pMainDlg;
+  m_pDbx = &dbox;
+  m_pMainWnd = m_pDbx;
 
-  // JHF : no tray icon and menu for PPC
-#if !defined(POCKET_PC)
   //HICON stIcon = app.LoadIcon(IDI_TRAY);
   //ASSERT(stIcon != NULL);
   m_LockedIcon = app.LoadIcon(IDI_LOCKEDICON);
   m_UnLockedIcon = app.LoadIcon(IDI_UNLOCKEDICON);
   int iData = prefs->GetPref(PWSprefs::ClosedTrayIconColour);
   SetClosedTrayIcon(iData);
-  m_pTrayIcon = new CSystemTray(m_pMainDlg, PWS_MSG_ICON_NOTIFY, L"PasswordSafe",
+  m_pTrayIcon = new CSystemTray(m_pDbx, PWS_MSG_ICON_NOTIFY, L"PasswordSafe",
                                 m_LockedIcon, dbox.m_RUEList,
                                 PWS_MSG_ICON_NOTIFY, IDR_POPTRAY);
   m_pTrayIcon->SetTarget(&dbox);
-#endif
 
   CLWnd ListenerWnd(dbox);
   if (SysInfo::IsUnderU3()) {
@@ -1214,23 +1193,20 @@ void ThisMfcApp::SetSystemTrayState(STATE s)
   }
 }
 
-#if !defined(POCKET_PC)
 //Copied from Knowledge Base article Q100770
-//But not for WinCE {kjp}
 BOOL ThisMfcApp::ProcessMessageFilter(int code, LPMSG lpMsg)
 {
   if (code < 0)
     CWinApp::ProcessMessageFilter(code, lpMsg);
 
-  if (m_bUseAccelerator && m_pMainDlg != NULL && m_ghAccelTable != NULL &&
+  if (m_bUseAccelerator && m_pDbx != NULL && m_ghAccelTable != NULL &&
       code != MSGF_MENU) {
-    if (::TranslateAccelerator(m_pMainDlg->m_hWnd, m_ghAccelTable, lpMsg))
+    if (::TranslateAccelerator(m_pDbx->m_hWnd, m_ghAccelTable, lpMsg))
       return TRUE;
   }
 
   return CWinApp::ProcessMessageFilter(code, lpMsg);
 }
-#endif
 
 // FindMenuItem() will find a menu item string from the specified
 // popup menu and returns its position (0-based) in the specified
@@ -1477,7 +1453,7 @@ void ThisMfcApp::GetLanguageFiles()
       szLanguage_Native = new wchar_t[inum + 1];
       ::GetLocaleInfo(lcid, LOCALE_SNATIVELANGNAME, szLanguage_Native, inum);
 
-      int jnum = 0, knum = 0;
+      int jnum = 0;
       if (!cs_CC.IsEmpty()) {
         // Get Country name in that language
         jnum = ::GetLocaleInfo(lcid, LOCALE_SNATIVECTRYNAME, NULL, 0);
@@ -1486,7 +1462,7 @@ void ThisMfcApp::GetLanguageFiles()
           ::GetLocaleInfo(lcid, LOCALE_SNATIVECTRYNAME, szCountry_Native, jnum);
         }
         // Get Country name in English
-        knum = ::GetLocaleInfo(lcid, LOCALE_SENGCOUNTRY, NULL, 0);
+        int knum = ::GetLocaleInfo(lcid, LOCALE_SENGCOUNTRY, NULL, 0);
         if (knum > 0) {
           szCountry_English = new wchar_t[knum + 1];
           ::GetLocaleInfo(lcid, LOCALE_SENGCOUNTRY, szCountry_English, knum);
