@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -14,42 +14,51 @@
 #include "WZPropertySheet.h"
 
 #include "Fonts.h"
-#include "TryAgainDlg.h"
 #include "SecString.h"
 #include "GeneralMsgBox.h"
 #include "PWFileDialog.h"
+#include "PKBaseDlg.h"
 #include "VirtualKeyboard/VKeyBoardDlg.h"
 
+#include "core/core.h" // for IDSC_UNKNOWN_ERROR
 #include "core/Util.h"
 #include "core/PWSdirs.h"
 #include "os/env.h"
 #include "os/file.h"
 #include "os/dir.h"
+#include "os/windows/yubi/YkLib.h"
 
 #include "resource3.h"
 
+#include <sstream>
 #include <iomanip>
 #include <shlobj.h>
 
-static wchar_t PSSWDCHAR = L'*';
+
+using namespace std;
+
+const wchar_t CWZSelectDB::PSSWDCHAR = L'*';
 
 IMPLEMENT_DYNAMIC(CWZSelectDB, CWZPropertyPage)
 
 CWZSelectDB::CWZSelectDB(CWnd *pParent, UINT nIDCaption, const int nType)
- : CWZPropertyPage(IDD, nIDCaption, nType), m_tries(0), m_state(0), m_pVKeyBoardDlg(NULL),
- m_bAdvanced(BST_UNCHECKED), m_bFileExistsUserAsked(false)
+ : CWZPropertyPage(IDD, nIDCaption, nType), m_tries(0), m_state(0),
+  m_pVKeyBoardDlg(NULL), m_bAdvanced(BST_UNCHECKED),
+  m_bFileExistsUserAsked(false),
+  m_filespec(L""), m_passkey(L""),
+  m_defexpdelim(L"\xbb"), m_pctlDB(new CEditExtn),
+  m_pctlPasskey(new CSecEditExtn)
 {
-  // Save pointer to my PropertySheet
   m_pWZPSH = (CWZPropertySheet *)pParent;
-
-  m_filespec = L"";
-  m_passkey = L"";
-  m_defexpdelim = L"\xbb";
-  m_pctlDB = new CEditExtn;
-  m_pctlPasskey = new CSecEditExtn;
-
   if (pws_os::getenv("PWS_PW_MODE", false) == L"NORMAL")
     m_pctlPasskey->SetSecure(false);
+  m_present = !IsYubiInserted(); // lie to trigger correct actions in timer event
+
+  // Call it as it also performs important initilisation
+  m_bVKAvailable = CVKeyBoardDlg::IsOSKAvailable();
+
+  m_bUseSecureDesktop = PWSprefs::GetInstance()->GetPref(PWSprefs::UseSecureDesktop);
+  m_iUserTimeLimit = PWSprefs::GetInstance()->GetPref(PWSprefs::SecureDesktopTimeout);
 }
 
 CWZSelectDB::~CWZSelectDB()
@@ -57,7 +66,7 @@ CWZSelectDB::~CWZSelectDB()
   delete m_pctlDB;
   delete m_pctlPasskey;
 
-  if (m_pVKeyBoardDlg != NULL) {
+  if (m_pVKeyBoardDlg != NULL && m_pVKeyBoardDlg->SaveKLID()) {
     // Save Last Used Keyboard
     UINT uiKLID = m_pVKeyBoardDlg->GetKLID();
     std::wostringstream os;
@@ -66,7 +75,6 @@ CWZSelectDB::~CWZSelectDB()
     StringX cs_KLID = os.str().c_str();
     PWSprefs::GetInstance()->SetPref(PWSprefs::LastUsedKeyboard, cs_KLID);
 
-    m_pVKeyBoardDlg->DestroyWindow();
     delete m_pVKeyBoardDlg;
   }
 }
@@ -75,15 +83,19 @@ void CWZSelectDB::DoDataExchange(CDataExchange* pDX)
 {
   CWZPropertyPage::DoDataExchange(pDX);
 
-  // Can't use DDX_Text for CSecEditExtn
-  m_pctlPasskey->DoDDX(pDX, m_passkey);
-
   //{{AFX_DATA_MAP(CWZSelectDB)
   DDX_Text(pDX, IDC_DATABASE, m_filespec);
 
-  DDX_Control(pDX, IDC_PASSKEY, *m_pctlPasskey);
   DDX_Control(pDX, IDC_DATABASE, *m_pctlDB);
   DDX_Check(pDX, IDC_ADVANCED, m_bAdvanced);
+  DDX_Control(pDX, IDC_SD_TOGGLE, m_ctlSDToggle);
+
+  // Can't use DDX_Text for CSecEditExtn
+  m_pctlPasskey->DoDDX(pDX, m_passkey);
+  DDX_Control(pDX, IDC_PASSKEY, *m_pctlPasskey);
+
+  DDX_Control(pDX, IDC_YUBI_PROGRESS, m_yubi_timeout);
+  DDX_Control(pDX, IDC_YUBI_STATUS, m_yubi_status);
 
   const UINT nID = m_pWZPSH->GetID();
 
@@ -103,7 +115,7 @@ void CWZSelectDB::DoDataExchange(CDataExchange* pDX)
   //}}AFX_DATA_MAP
 }
 
-void AFXAPI CWZSelectDB::DDV_CheckExpDelimiter(CDataExchange* pDX, 
+void AFXAPI CWZSelectDB::DDV_CheckExpDelimiter(CDataExchange* pDX,
                                                const CString &delimiter)
 {
   if (pDX->m_bSaveAndValidate) {
@@ -112,7 +124,7 @@ void AFXAPI CWZSelectDB::DDV_CheckExpDelimiter(CDataExchange* pDX,
       gmb.AfxMessageBox(IDS_NEEDDELIMITER);
       pDX->Fail();
       return;
-    }   
+    }
     if (delimiter[0] == '"') {
       CGeneralMsgBox gmb;
       gmb.AfxMessageBox(IDS_INVALIDDELIMITER);
@@ -124,6 +136,7 @@ void AFXAPI CWZSelectDB::DDV_CheckExpDelimiter(CDataExchange* pDX,
 BEGIN_MESSAGE_MAP(CWZSelectDB, CWZPropertyPage)
   //{{AFX_MSG_MAP(CWZSelectDB)
   ON_WM_CTLCOLOR()
+  ON_WM_TIMER()
   ON_EN_CHANGE(IDC_PASSKEY, OnPassKeyChange)
   ON_EN_CHANGE(IDC_DATABASE, OnDatabaseChange)
   ON_BN_CLICKED(IDC_BTN_BROWSE, OnOpenFileBrowser)
@@ -131,6 +144,9 @@ BEGIN_MESSAGE_MAP(CWZSelectDB, CWZPropertyPage)
   ON_MESSAGE(PWS_MSG_INSERTBUFFER, OnInsertBuffer)
   ON_BN_CLICKED(ID_HELP, OnHelp)
   ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
+  ON_BN_CLICKED(IDC_ENTERCOMBINATION, OnEnterCombination)
+  ON_BN_CLICKED(IDC_YUBIKEY_BTN, OnYubikeyBtn)
+  ON_BN_CLICKED(IDC_SD_TOGGLE, OnSwitchSecureDesktop)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -143,9 +159,11 @@ BOOL CWZSelectDB::OnInitDialog()
 {
   CWZPropertyPage::OnInitDialog();
 
-  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
+  // Setup a timer to poll YubiKey every 250 ms
+  SetTimer(1, 250, 0);
 
-  m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
+  // Configure dialog for Secure Dialog
+  ConfigureDialog();
 
   const UINT nID = m_pWZPSH->GetID();
   CString cs_text,cs_temp;
@@ -224,16 +242,49 @@ BOOL CWZSelectDB::OnInitDialog()
     GetDlgItem(IDC_VKB)->EnableWindow(FALSE);
   }
 
-  // Disble passphrase until database name filled in
-  m_pctlPasskey->EnableWindow(TRUE);
-
-  // Disable Next until fields set
+   // Disable Next until fields set
   m_pWZPSH->SetWizardButtons(0);
 
   CString cs_tmp(MAKEINTRESOURCE(m_pWZPSH->GetButtonID()));
   m_pWZPSH->GetDlgItem(ID_WIZNEXT)->SetWindowText(cs_tmp);
 
   GetDlgItem(IDC_DATABASE)->SetFocus();
+
+  // Yubi-related initializations:
+  m_yubiLogo.LoadBitmap(IDB_YUBI_LOGO);
+  m_yubiLogoDisabled.LoadBitmap(IDB_YUBI_LOGO_DIS);
+
+  if (!m_bUseSecureDesktop) {
+    // Disable passphrase until database name filled in
+    m_pctlPasskey->EnableWindow(TRUE);
+
+    CWnd *ybn = GetDlgItem(IDC_YUBIKEY_BTN);
+
+    if (YubiExists()) {
+      ybn->ShowWindow(SW_SHOW);
+      m_yubi_status.ShowWindow(SW_SHOW);
+    }
+    else {
+      ybn->ShowWindow(SW_HIDE);
+      m_yubi_status.ShowWindow(SW_HIDE);
+    }
+    m_yubi_timeout.ShowWindow(SW_HIDE);
+    m_yubi_timeout.SetRange(0, 15);
+    bool bYubiInserted = IsYubiInserted();
+    // MFC has ancient bug: can't render diasbled version of bitmap,
+    // so instead of showing drek, we roll our own, and leave enabled.
+    ybn->EnableWindow(TRUE);
+
+    if (bYubiInserted) {
+      ((CButton*)ybn)->SetBitmap(m_yubiLogo);
+      m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_CLICK_PROMPT)));
+    }
+    else {
+      ((CButton*)ybn)->SetBitmap(m_yubiLogoDisabled);
+      m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_INSERT_PROMPT)));
+    }
+  }
+
   return FALSE;
 }
 
@@ -280,6 +331,63 @@ HBRUSH CWZSelectDB::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
   return hbr;
 }
 
+void CWZSelectDB::OnSwitchSecureDesktop()
+{
+  m_bUseSecureDesktop = !m_bUseSecureDesktop;
+
+  ConfigureDialog();
+}
+
+void CWZSelectDB::ConfigureDialog()
+{
+  // This bit makes the background come out right on the bitmaps - these 2 bitmaps use white as the mask
+  m_ctlSDToggle.SetBitmapMaskAndID(RGB(255, 255, 255), m_bUseSecureDesktop ? IDB_USING_SD : IDB_NOT_USING_SD);
+  m_ctlSDToggle.Invalidate();
+
+  if (!m_bUseSecureDesktop)
+  {
+    Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
+    m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
+
+    GetDlgItem(IDC_ENTERCOMBINATION)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_ENTERCOMBINATION)->EnableWindow(FALSE);
+    GetDlgItem(IDC_STATIC_ENTERCOMBINATION)->ShowWindow(SW_SHOW);
+
+    GetDlgItem(IDC_PASSKEY)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_PASSKEY)->EnableWindow(TRUE);
+    GetDlgItem(IDC_VKB)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_VKB)->EnableWindow(TRUE);
+
+    if (IsYubiInserted())
+    {
+      GetDlgItem(IDC_YUBIKEY_BTN)->ShowWindow(SW_SHOW);
+      GetDlgItem(IDC_YUBIKEY_BTN)->EnableWindow(TRUE);
+      GetDlgItem(IDC_YUBI_PROGRESS)->ShowWindow(SW_SHOW);
+      GetDlgItem(IDC_YUBI_PROGRESS)->EnableWindow(TRUE);
+      GetDlgItem(IDC_YUBI_STATUS)->ShowWindow(SW_SHOW);
+      GetDlgItem(IDC_YUBI_STATUS)->EnableWindow(TRUE);
+    }
+  }
+  else
+  {
+    GetDlgItem(IDC_ENTERCOMBINATION)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_ENTERCOMBINATION)->EnableWindow(TRUE);
+    GetDlgItem(IDC_STATIC_ENTERCOMBINATION)->ShowWindow(SW_HIDE);
+
+    GetDlgItem(IDC_PASSKEY)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_PASSKEY)->EnableWindow(FALSE);
+    GetDlgItem(IDC_VKB)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_VKB)->EnableWindow(FALSE);
+
+    GetDlgItem(IDC_YUBIKEY_BTN)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_YUBIKEY_BTN)->EnableWindow(FALSE);
+    GetDlgItem(IDC_YUBI_PROGRESS)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_YUBI_PROGRESS)->EnableWindow(FALSE);
+    GetDlgItem(IDC_YUBI_STATUS)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_YUBI_STATUS)->EnableWindow(FALSE);
+  }
+}
+
 BOOL CWZSelectDB::OnSetActive()
 {
   CWZPropertyPage::OnSetActive();
@@ -300,6 +408,34 @@ void CWZSelectDB::OnAdvanced()
   m_pWZPSH->GetDlgItem(ID_WIZNEXT)->SetWindowText(cs_tmp);
 }
 
+void CWZSelectDB::OnEnterCombination()
+{
+  // Only needed for its thread processing - never dispays its own dialog (no DoModal etc.)
+  CPKBaseDlg PKBaseDlg(IDD_PASSKEYENTRY_SD, this, true);
+
+  // Avoid polling Yubikey from > 1 thread
+  m_yubiPollDisable = true;
+
+  // Get current monitor - not possible in PKBaseDlg as its dialog is not shown
+  HMONITOR hCurrentMonitor = MonitorFromWindow(this->GetSafeHwnd(), MONITOR_DEFAULTTONEAREST);
+
+  // Get passphrase from Secure Desktop
+  PKBaseDlg.StartThread(IDD_SDGETPHRASE, hCurrentMonitor);
+
+  if (PKBaseDlg.m_GMP.bPhraseEntered) {
+    m_passkey = PKBaseDlg.m_GMP.sPhrase.c_str();
+    if (m_passkey.GetLength() > 0)
+      m_state |= KEYPRESENT;
+    else
+      m_state &= ~KEYPRESENT;
+
+    m_pWZPSH->SetWizardButtons(m_state == BOTHPRESENT ? PSWIZB_NEXT : 0);
+  }
+
+  ShowWindow(SW_SHOW);
+  UpdateData(FALSE);
+}
+
 void CWZSelectDB::OnPassKeyChange()
 {
   CString cs_Passkey;
@@ -375,7 +511,7 @@ LRESULT CWZSelectDB::OnWizardNext()
         std::wstring dir, cdrive, cdir, cfile, cextn;
         pws_os::splitpath(m_filespec.GetString(), cdrive, cdir, cfile, cextn);
         //  If root directory - nothing to create
-        if (cdir == L"\\")          
+        if (cdir == L"\\")
           break;
 
         // Try and create necessary directories
@@ -432,32 +568,24 @@ LRESULT CWZSelectDB::OnWizardNext()
 
 bool CWZSelectDB::ProcessPhrase(const StringX &filename, const StringX &passkey)
 {
-  PWScore tmpcore;
+  PWSAuxCore tmpcore; // doesn't really need to be PWSAuxCore, since we only check the passkey
   if (m_pWZPSH->WZPSHCheckPasskey(filename, passkey,
                                   &tmpcore) == PWScore::SUCCESS) {
     m_tries = 0;
     return true;
-  }
-
-  if (m_tries >= 2) {
-    CTryAgainDlg errorDlg(this);
-
-    INT_PTR nResponse = errorDlg.DoModal();
-    if (nResponse == IDCANCEL) {
-      int status = errorDlg.GetCancelReturnValue();
-      if (status == TAR_OPEN) { // open another
-        PostMessage(WM_COMMAND, IDC_BTN_BROWSE);
-        return false;
-      }
-    }
   } else {
-    m_tries++;
     CGeneralMsgBox gmb;
-    gmb.AfxMessageBox(IDS_INCORRECTKEY);
+    if (m_tries++ >= 2) {
+      CString cs_toomany;
+      cs_toomany.Format(IDS_TOOMANYTRIES, m_tries);
+      gmb.AfxMessageBox(cs_toomany);
+    } else {
+      gmb.AfxMessageBox(IDS_INCORRECTKEY);
+    }
     m_pctlPasskey->SetSel(MAKEWORD(-1, 0));
     m_pctlPasskey->SetFocus();
+    return false;
   }
-  return false;
 }
 
 void CWZSelectDB::OnOpenFileBrowser()
@@ -504,7 +632,7 @@ void CWZSelectDB::OnOpenFileBrowser()
 
   std::wstring ExportFileName = PWSUtil::GetNewFileName(m_pWZPSH->WZPSHGetCurFile().c_str(),
                                                         CString::PCXSTR(cs_suffix));
-  CPWFileDialog fd(bTYPE_OPEN, cs_suffix, uimsgid != IDS_CHOOSEDATABASE ? ExportFileName.c_str() : NULL, 
+  CPWFileDialog fd(bTYPE_OPEN, cs_suffix, uimsgid != IDS_CHOOSEDATABASE ? ExportFileName.c_str() : NULL,
                    dwflags, cs_filter, this);
 
   fd.m_ofn.lpstrTitle = cs_text;
@@ -539,9 +667,11 @@ void CWZSelectDB::OnOpenFileBrowser()
   if (rc == IDOK) {
     m_filespec = fd.GetPathName();
     m_pctlDB->SetWindowText(m_filespec);
-    m_pctlPasskey->EnableWindow(TRUE);
-    if (m_pctlPasskey->IsWindowEnabled() == TRUE) {
-      m_pctlPasskey->SetFocus();
+    if (!m_bUseSecureDesktop) {
+      m_pctlPasskey->EnableWindow(TRUE);
+      if (m_pctlPasskey->IsWindowEnabled() == TRUE) {
+        m_pctlPasskey->SetFocus();
+      }
     }
     // If the file exists and we are doing a save, CFileDialog
     // would have prompted the user
@@ -552,29 +682,38 @@ void CWZSelectDB::OnOpenFileBrowser()
 
 void CWZSelectDB::OnVirtualKeyboard()
 {
+  // This is used if Secure Desktop isn't!
+  DWORD dwError; //  Define it here to stop warning that local variable is initialized but not referenced later on
+
   // Shouldn't be here if couldn't load DLL. Static control disabled/hidden
   if (!CVKeyBoardDlg::IsOSKAvailable())
     return;
 
-  if (m_pVKeyBoardDlg != NULL && m_pVKeyBoardDlg->IsWindowVisible()) {
+  if (m_hwndVKeyBoard != NULL && ::IsWindowVisible(m_hwndVKeyBoard)) {
     // Already there - move to top
-    m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    ::SetWindowPos(m_hwndVKeyBoard, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
     return;
   }
 
   // If not already created - do it, otherwise just reset it
   if (m_pVKeyBoardDlg == NULL) {
     StringX cs_LUKBD = PWSprefs::GetInstance()->GetPref(PWSprefs::LastUsedKeyboard);
-    m_pVKeyBoardDlg = new CVKeyBoardDlg(this, cs_LUKBD.c_str());
-    m_pVKeyBoardDlg->Create(CVKeyBoardDlg::IDD);
-  } else {
+    m_pVKeyBoardDlg = new CVKeyBoardDlg(this->GetSafeHwnd(), this->GetSafeHwnd(), cs_LUKBD.c_str());
+    m_hwndVKeyBoard = CreateDialogParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_SDVKEYBOARD), this->GetSafeHwnd(),
+      (DLGPROC)(m_pVKeyBoardDlg->VKDialogProc), (LPARAM)(m_pVKeyBoardDlg));
+
+    if (m_hwndVKeyBoard == NULL) {
+      dwError = pws_os::IssueError(_T("CreateDialogParam - IDD_SDVKEYBOARD"), false);
+      ASSERT(m_hwndVKeyBoard);
+    }
+  }
+  else {
     m_pVKeyBoardDlg->ResetKeyboard();
   }
 
-  // Now show it and make it top
-  m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
-
-  return;
+  // Now show it and make it top & enable it
+  ::SetWindowPos(m_hwndVKeyBoard, HWND_TOP, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
+  ::EnableWindow(m_hwndVKeyBoard, TRUE);
 }
 
 LRESULT CWZSelectDB::OnInsertBuffer(WPARAM, LPARAM)
@@ -596,12 +735,113 @@ LRESULT CWZSelectDB::OnInsertBuffer(WPARAM, LPARAM)
   // Insert the buffer
   m_passkey.Insert(nStartChar, vkbuffer);
 
+  // Put cursor at end of inserted text
+  m_pctlPasskey->SetSel(nStartChar + vkbuffer.GetLength(),
+    nStartChar + vkbuffer.GetLength());
+
   // Update the dialog
   UpdateData(FALSE);
 
-  // Put cursor at end of inserted text
-  m_pctlPasskey->SetSel(nStartChar + vkbuffer.GetLength(),
-                        nStartChar + vkbuffer.GetLength());
+  // Ensure flags set so that buttons are activated as required
+  if (vkbuffer.GetLength() > 0)
+    OnPassKeyChange();
 
   return 0L;
 }
+
+void CWZSelectDB::yubiInserted(void)
+{
+  CButton *ybn = (CButton *)GetDlgItem(IDC_YUBIKEY_BTN);
+  ybn->SetBitmap(m_yubiLogo);
+  m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_CLICK_PROMPT)));
+  // In case this is the first time:
+  ybn->ShowWindow(SW_SHOW);
+  m_yubi_status.ShowWindow(SW_SHOW);
+}
+
+void CWZSelectDB::yubiRemoved(void)
+{
+  ((CButton*)GetDlgItem(IDC_YUBIKEY_BTN))->SetBitmap(m_yubiLogoDisabled);
+  m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_INSERT_PROMPT)));
+}
+  BYTE respBuf[SHA1_DIGEST_SIZE];
+
+void CWZSelectDB::yubiShowChallengeSent()
+{
+  // A request's in the air, setup GUI to wait for reply
+  m_yubi_status.ShowWindow(SW_HIDE);
+  m_yubi_status.SetWindowText(_T(""));
+  m_yubi_timeout.ShowWindow(SW_SHOW);
+  m_yubi_timeout.SetPos(15);
+}
+
+void CWZSelectDB::yubiProcessCompleted(YKLIB_RC yrc, unsigned short ts, const BYTE *respBuf)
+{
+  switch (yrc) {
+  case YKLIB_OK:
+    m_yubi_status.ShowWindow(SW_SHOW);
+    m_yubi_timeout.ShowWindow(SW_HIDE);
+    m_yubi_timeout.SetPos(0);
+    m_yubi_status.SetWindowText(_T(""));
+    TRACE(_T("yubiCheckCompleted: YKLIB_OK"));
+    m_pending = false;
+    m_passkey = Bin2Hex(respBuf, SHA1_DIGEST_SIZE);
+    // The returned hash is the passkey
+    m_pWZPSH->SetWizardButtons(PSWIZB_NEXT); // enable
+    // This will check the password, etc.:
+    UpdateData(FALSE); // passwd -> control
+    m_pWZPSH->PostMessage(WM_COMMAND, MAKELONG(ID_WIZNEXT, BN_CLICKED), 0);
+    break;
+  case YKLIB_PROCESSING:  // Still processing or waiting for the result
+    break;
+  case YKLIB_TIMER_WAIT:  // A given number of seconds remain
+    m_yubi_timeout.SetPos(ts);
+    break;
+
+  case YKLIB_INVALID_RESPONSE:  // Invalid or no response
+    m_pending = false;
+    m_yubi_timeout.ShowWindow(SW_HIDE);
+    m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_TIMEOUT)));
+    m_yubi_status.ShowWindow(SW_SHOW);
+    break;
+
+  default:                // A non-recoverable error has occured
+    m_pending = false;
+    m_yubi_timeout.ShowWindow(SW_HIDE);
+    m_yubi_status.ShowWindow(SW_SHOW);
+    // Generic error message
+    TRACE(_T("yubiCompleted(%d)\n"), yrc);
+    m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDSC_UNKNOWN_ERROR)));
+    break;
+  }
+}
+
+void CWZSelectDB::OnYubikeyBtn()
+{
+  UpdateData(TRUE);
+  yubiRequestHMACSha1(m_passkey);
+}
+
+void CWZSelectDB::OnTimer(UINT_PTR)
+{
+  // Ignore if Secure Desktop
+  if (m_bUseSecureDesktop)
+    return;
+
+  // If an operation is pending, check if it has completed
+  if (m_pending) {
+    yubiCheckCompleted();
+  } else {
+    // No HMAC operation is pending - check if one and only one key is present
+    bool inserted = IsYubiInserted();
+    // call relevant callback if something's changed
+    if (inserted != m_present) {
+      m_present = inserted;
+      if (m_present) {
+        CPKBaseDlg::SetYubiExists();
+        yubiInserted();
+      } else
+        yubiRemoved();
+    }
+  }
+}
