@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -34,25 +34,17 @@
  * in case of failure.  Returns PWScore::SUCCESS on success
  */
 
-int ReadCore(PWScore& othercore, const wxString& file, const StringX& combination,
-                bool showMsgbox /*= true*/, wxWindow* msgboxParent /*= NULL*/,
-				bool setupCopy /*= false*/)
+int ReadCore(PWSAuxCore& othercore, const wxString& file, const StringX& combination,
+             bool showMsgbox /*= true*/, wxWindow* msgboxParent /*= NULL*/,
+        bool setupCopy /*= false*/)
 {
   othercore.ClearData();
 
-  // Reading a new file changes the preferences!
-  const StringX sxSavePrefString(PWSprefs::GetInstance()->Store());
-  const bool bDBPrefsChanged = PWSprefs::GetInstance()->IsDBprefsChanged();
-
   StringX dbpath(tostringx(file));
   int rc = othercore.ReadFile(dbpath, combination);
 
   if (setupCopy)
-	  PWSprefs::GetInstance()->SetupCopyPrefs();
-
-  // Reset database preferences - first to defaults then add saved changes!
-  PWSprefs::GetInstance()->Load(sxSavePrefString);
-  PWSprefs::GetInstance()->SetDBprefsChanged(bDBPrefsChanged);
+    PWSprefs::GetInstance()->SetupCopyPrefs();
 
   switch (rc) {
     case PWScore::SUCCESS:
@@ -188,4 +180,29 @@ bool MultiCheckboxValidator::Validate(wxWindow* parent)
   }
 }
 
+void ShowHideText(wxTextCtrl *&txtCtrl, const wxString &text,
+                  wxSizer *sizer, bool show)
+{
+  wxWindow *parent = txtCtrl->GetParent();
+  wxWindowID id = txtCtrl->GetId();
+  wxValidator *validator = txtCtrl->GetValidator();
+
+  // Per Dave Silvia's suggestion:
+  // Following kludge since wxTE_PASSWORD style is immutable
+  wxTextCtrl *tmp = txtCtrl;
+  txtCtrl = new wxTextCtrl(parent, id, text,
+                           wxDefaultPosition, wxDefaultSize,
+                           show ? 0 : wxTE_PASSWORD);
+  if (validator != NULL)
+    txtCtrl->SetValidator(*validator);
+  ApplyPasswordFont(txtCtrl);
+  sizer->Replace(tmp, txtCtrl);
+  delete tmp;
+  sizer->Layout();
+  if (!text.IsEmpty()) {
+    txtCtrl->ChangeValue(text);
+    txtCtrl->SetModified(true);
+  }
+}
+
 int pless(int* first, int* second) { return *first - *second; }
